import pytest

from verbecc.src.mlconjug import mlconjug
from verbecc.src.inflectors.lang.inflector_fr import InflectorFr
from verbecc.src.defs.constants import config


@pytest.fixture(scope="module")
def verb_template_pairs():
    inf = InflectorFr()
    yield [(v.infinitive, v.template) for v in inf._verb_parser.verbs]


def test_extract_verb_features():
    if config.ml:
        assert mlconjug.extract_verb_features("parler", "fr", (2, 7)) == [
            "END=er",
            "END=ler",
            "END=rler",
            "END=arler",
            "END=parler",
            "START=pa",
            "START=par",
            "START=parl",
            "START=parle",
            "START=parler",
            "LEN=6",
            "VOW_NUM=2",
            "CONS_NUM=4",
            "V/C=0.5",
        ]


def test_DataSet_construct_dict_conjug(verb_template_pairs):
    if config.ml:
        dict_conjug = mlconjug.DataSet(verb_template_pairs).dict_conjug
        assert "abaisser" in dict_conjug["aim:er"]


def test_DataSet_split_test_train(verb_template_pairs):
    if config.ml:
        data_set = mlconjug.DataSet(verb_template_pairs)
        assert data_set.min_threshold == 8
        assert data_set.split_proportion == 0.5
        assert len(data_set.train_input) == len(data_set.train_labels)
        assert len(data_set.test_input) == len(data_set.test_labels)
        for verb in data_set.test_input:
            assert verb not in data_set.train_input
        test_verb = data_set.test_input[0]
        train_verb = data_set.train_input[0]
        test_template = next(p[1] for p in verb_template_pairs if p[0] == test_verb)
        train_template = next(p[1] for p in verb_template_pairs if p[0] == train_verb)
        assert (
            data_set.templates[
                data_set.test_labels[data_set.test_input.index(test_verb)]
            ]
            == test_template
        )
        assert (
            data_set.templates[
                data_set.train_labels[data_set.train_input.index(train_verb)]
            ]
            == train_template
        )


def test_mlconjug_template_predictor(verb_template_pairs):
    if config.ml:
        predictor = mlconjug.TemplatePredictor(verb_template_pairs, lang="fr")
        template, prediction_score = predictor.predict("parler")
        assert template == "aim:er"
        assert prediction_score > 0.97



from lxml import etree
from typing import List

from verbecc.src.defs.types.person import Person


class PersonEnding:
    """
    p_elem
    aka <p>
    Ending for a specific verb template, mood, tense and grammatical person
    May also have an alternate ending for an alternative spelling.
    E.g. Ending for aim:er indicatif présent 2nd Person Plural = ['ez']
    E.g. Ending for pa:yer indicatif présent 1st Person Singular = ['ie', 'ye']
    Explanation: 'ye' is an alternate spelling (je paie, je paye)
    p_elem
        Example p_elems:
            <p><i>ez</i></p>
            <p><i>eoir</i><i>oir</i></p>
            <p></p>

    person
    A grammar_defines.PERSONS value indicating which person
    this PersonEnding is for, e.g. for aim:er, "ez" is '2p' (second person plural)
    """

    def __init__(self, p_elem: etree._Element, person: Person) -> None:
        self.person = person
        self.endings: List[str] = []
        for i_elem in p_elem.findall("i", None):
            ending = str("")
            if i_elem.text is not None:
                ending += str(i_elem.text)
            self.endings.append(ending)

    def get_person(self) -> Person:
        return self.person

    def get_endings(self) -> List[str]:
        return self.endings

    def get_ending(self) -> str:
        return self.endings[0]

    def get_alternate_ending_if_available(self) -> str:
        if len(self.endings) > 1:
            return self.endings[1]
        return self.endings[0]

    def __repr__(self) -> str:
        return "person={} endings={}".format(self.person, self.endings)



from verbecc.src.conjugator.conjugator import Conjugator
from verbecc.src.defs.types.alternates_behavior import AlternatesBehavior
from verbecc.src.defs.types.conjugation import (
    MoodsConjugation,
    MoodConjugation,
    TenseConjugation,
    PersonConjugation,
    ConjugationInfo,
    Conjugation,
)
from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.person import Person, is_plural, is_singular
from verbecc.src.defs.types.partiple_inflection import ParticipleInflection
from verbecc.src.defs.types.lang_code import LangCodeISO639_1
from verbecc.src.defs.types.mood import (
    Mood,
    MoodCa,
    MoodEs,
    MoodFr,
    MoodIt,
    MoodPt,
    MoodRo,
)
from verbecc.src.defs.types.tense import (
    Tense,
    TenseCa,
    TenseEs,
    TenseFr,
    TenseIt,
    TensePt,
    TenseRo,
)
from verbecc.src.defs.types.lang_specific_options import LangSpecificOptions
from verbecc.src.defs.types.lang.es.lang_specific_options_es import (
    LangSpecificOptionsEs,
)
from verbecc.src.defs.types.lang.es.voseo_options import VoseoOptions
import verbecc.src.defs.constants.localization as localization
import verbecc.src.defs.constants.grammar_defines as grammar_defines
from verbecc.src.parsers.mood_template import MoodTemplate
from verbecc.src.parsers.verb import Verb
from verbecc.src.defs.types.exceptions import (
    ConjugatorError,
    ConjugationTemplateError,
    ConjugationsParserError,
    VerbNotFoundError,
    TemplateNotFoundError,
    InvalidLangError,
    InvalidMoodError,
    InvalidTenseError,
)



from lxml import etree
from typing import Dict

from verbecc.src.defs.types.lang_code import LangCodeISO639_1 as Lang
from verbecc.src.parsers.tense_template import TenseTemplate


class MoodTemplate:
    def __init__(self, lang: Lang, mood_elem: etree._Element) -> None:
        self.lang = lang
        self.name = mood_elem.tag.lower()
        self.tense_templates: Dict[str, TenseTemplate] = {}
        for tense_elem in mood_elem:  # type: ignore
            self.tense_templates[tense_elem.tag] = TenseTemplate(
                lang=lang, mood=self.name, tense_elem=tense_elem
            )

    def __repr__(self) -> str:
        return "name={} tense_templates={}".format(self.name, self.tense_templates)



from verbecc.src.conjugator.conjugator import Conjugator, SUPPORTED_LANGUAGES


def train_models() -> None:
    print("Begin model training")
    for i, l in enumerate(SUPPORTED_LANGUAGES.keys()):
        print(f"Training model {i+1} of {len(SUPPORTED_LANGUAGES.keys())} lang={l}")
        print("Please be patient, this could take a while...")
        cg = Conjugator(lang=l)
        if l == "fr":
            c = cg.conjugate("etre")
        elif l == "it":
            c = cg.conjugate("essere")
        elif l == "ro":
            c = cg.conjugate("fi")
        else:
            c = cg.conjugate("ser")
        print(f"lang={l} {c}")
        print(f"Finished training model lang={l}")
    print("Model training complete")



import pytest
from lxml import etree

from verbecc.src.defs.types.lang_code import LangCodeISO639_1 as Lang
from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.person import Person
from verbecc.src.conjugator.conjugator import Conjugator
from verbecc.src.parsers.tense_template import TenseTemplate
from verbecc.src.defs.types.alternates_behavior import AlternatesBehavior


@pytest.fixture(scope="module")
def cg():
    cg = Conjugator(lang="ca")
    yield cg


def test_all_verbs_have_templates(cg):
    verbs = cg.get_verbs()
    assert len(verbs) == 8616
    template_names = cg.get_template_names()
    assert len(template_names) == 66
    missing_verbs = []
    missing_templates = set()
    for verb in verbs:
        if verb.template not in template_names:
            missing_templates.add(verb.template)
            missing_verbs.append(verb)
    assert len(missing_templates) == len(missing_verbs) == 0


def test_find_verb_by_infinitive(cg):
    v = cg.find_verb_by_infinitive("abandonar")
    assert v.infinitive == "abandonar"
    assert v.template == "cant:ar"


@pytest.mark.parametrize(
    "infinitive,mood,tense,alternates_behavior,gender,expected_result",
    [
        (
            "ser",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo sóc",
                "tu ets",
                "ell és",
                "nosaltres som",
                "vosaltres sou",
                "ells són",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo era",
                "tu eres",
                "ell era",
                "nosaltres érem",
                "vosaltres éreu",
                "ells eren",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo fui",
                "tu fores",
                "ell fou",
                "nosaltres fórem",
                "vosaltres fóreu",
                "ells foren",
            ],
        ),
        (
            "ser",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo seré",
                "tu seràs",
                "ell serà",
                "nosaltres serem",
                "vosaltres sereu",
                "ells seran",
            ],
        ),
        (
            "ser",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo sigui",
                "tu siguis",
                "ell sigui",
                "nosaltres siguem",
                "vosaltres sigueu",
                "ells siguin",
            ],
        ),
        (
            "ser",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo fos",
                "tu fossis",
                "ell fos",
                "nosaltres fóssim",
                "vosaltres fóssiu",
                "ells fossin",
            ],
        ),
        (
            "ser",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo seria",
                "tu series",
                "ell seria",
                "nosaltres seríem",
                "vosaltres seríeu",
                "ells serien",
            ],
        ),
        (
            "ser",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["sigues", "sigui", "siguem", "sigueu", "siguin"],
        ),
        (
            "parlar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlo",
                "tu parles",
                "ell parla",
                "nosaltres parlem",
                "vosaltres parleu",
                "ells parlen",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlava",
                "tu parlaves",
                "ell parlava",
                "nosaltres parlàvem",
                "vosaltres parlàveu",
                "ells parlaven",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlí",
                "tu parlares",
                "ell parlà",
                "nosaltres parlàrem",
                "vosaltres parlàreu",
                "ells parlaren",
            ],
        ),
        (
            "parlar",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlaré",
                "tu parlaràs",
                "ell parlarà",
                "nosaltres parlarem",
                "vosaltres parlareu",
                "ells parlaran",
            ],
        ),
        (
            "parlar",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parli",
                "tu parlis",
                "ell parli",
                "nosaltres parlem",
                "vosaltres parleu",
                "ells parlin",
            ],
        ),
        (
            "parlar",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlés",
                "tu parlessis",
                "ell parlés",
                "nosaltres parléssim",
                "vosaltres parléssiu",
                "ells parlessin",
            ],
        ),
        (
            "parlar",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parlaria",
                "tu parlaries",
                "ell parlaria",
                "nosaltres parlaríem",
                "vosaltres parlaríeu",
                "ells parlarien",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estic",
                "tu estàs",
                "ell està",
                "nosaltres estem",
                "vosaltres esteu",
                "ells estan",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estava",
                "tu estaves",
                "ell estava",
                "nosaltres estàvem",
                "vosaltres estàveu",
                "ells estaven",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estiguí",
                "tu estigueres",
                "ell estigué",
                "nosaltres estiguérem",
                "vosaltres estiguéreu",
                "ells estigueren",
            ],
        ),
        (
            "estar",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estaré",
                "tu estaràs",
                "ell estarà",
                "nosaltres estarem",
                "vosaltres estareu",
                "ells estaran",
            ],
        ),
        (
            "estar",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estigui",
                "tu estiguis",
                "ell estigui",
                "nosaltres estiguem",
                "vosaltres estigueu",
                "ells estiguin",
            ],
        ),
        (
            "estar",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estigués",
                "tu estiguessis",
                "ell estigués",
                "nosaltres estiguéssim",
                "vosaltres estiguéssiu",
                "ells estiguessin",
            ],
        ),
        (
            "estar",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo estaria",
                "tu estaries",
                "ell estaria",
                "nosaltres estaríem",
                "vosaltres estaríeu",
                "ells estarien",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo he",
                "tu has",
                "ell ha",
                "nosaltres havem",
                "vosaltres haveu",
                "ells han",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo havia",
                "tu havies",
                "ell havia",
                "nosaltres havíem",
                "vosaltres havíeu",
                "ells havien",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo haguí",
                "tu hagueres",
                "ell hagué",
                "nosaltres haguérem",
                "vosaltres haguéreu",
                "ells hagueren",
            ],
        ),
        (
            "haver",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo hauré",
                "tu hauràs",
                "ell haurà",
                "nosaltres haurem",
                "vosaltres haureu",
                "ells hauran",
            ],
        ),
        (
            "haver",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo hauria",
                "tu hauries",
                "ell hauria",
                "nosaltres hauríem",
                "vosaltres hauríeu",
                "ells haurien",
            ],
        ),
        (
            "haver",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo hagi",
                "tu hagis",
                "ell hagi",
                "nosaltres hàgim",
                "vosaltres hàgiu",
                "ells hagin",
            ],
        ),
        (
            "tenir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo tinc",
                "tu tens",
                "ell té",
                "nosaltres tenim",
                "vosaltres teniu",
                "ells tenen",
            ],
        ),
        (
            "fer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo faig",
                "tu fas",
                "ell fa",
                "nosaltres fem",
                "vosaltres feu",
                "ells fan",
            ],
        ),
        (
            "fer",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo feia",
                "tu feies",
                "ell feia",
                "nosaltres fèiem",
                "vosaltres fèieu",
                "ells feien",
            ],
        ),
        (
            "servir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo serveixo",
                "tu serveixes",
                "ell serveix",
                "nosaltres servim",
                "vosaltres serviu",
                "ells serveixen",
            ],
        ),
        (
            "veure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo veig",
                "tu veus",
                "ell veu",
                "nosaltres veiem",
                "vosaltres veieu",
                "ells veuen",
            ],
        ),
        (
            "abandonar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo abandono",
                "tu abandones",
                "ell abandona",
                "nosaltres abandonem",
                "vosaltres abandoneu",
                "ells abandonen",
            ],
        ),
        (
            "rebre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo rebo",
                "tu reps",
                "ell rep",
                "nosaltres rebem",
                "vosaltres rebeu",
                "ells reben",
            ],
        ),
        (
            "cabre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo cabo",
                "tu caps",
                "ell cap",
                "nosaltres cabem",
                "vosaltres cabeu",
                "ells caben",
            ],
        ),
        (
            "començar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo començo",
                "tu comences",
                "ell comença",
                "nosaltres comencem",
                "vosaltres comenceu",
                "ells comencen",
            ],
        ),
        (
            "enaiguar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo enaiguo",
                "tu enaigües",
                "ell enaigua",
                "nosaltres enaigüem",
                "vosaltres enaigüeu",
                "ells enaigüen",
            ],
        ),
        (
            "envejar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo envejo",
                "tu enveges",
                "ell enveja",
                "nosaltres envegem",
                "vosaltres envegeu",
                "ells envegen",
            ],
        ),
        (
            "esglaiar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo esglaio",
                "tu esglaies",
                "ell esglaia",
                "nosaltres esglaiem",
                "vosaltres esglaieu",
                "ells esglaien",
            ],
        ),
        (
            "trencar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo trenco",
                "tu trenques",
                "ell trenca",
                "nosaltres trenquem",
                "vosaltres trenqueu",
                "ells trenquen",
            ],
        ),
        (
            "perdre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo perdo",
                "tu perds",
                "ell perd",
                "nosaltres perdem",
                "vosaltres perdeu",
                "ells perden",
            ],
        ),
        (
            "caure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo caic",
                "tu caus",
                "ell cau",
                "nosaltres caiem",
                "vosaltres caieu",
                "ells cauen",
            ],
        ),
        (
            "collar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo collo",
                "tu colles",
                "ell colla",
                "nosaltres collem",
                "vosaltres colleu",
                "ells collen",
            ],
        ),
        (
            "collir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo cullo",
                "tu culls",
                "ell cull",
                "nosaltres collim",
                "vosaltres colliu",
                "ells cullen",
            ],
        ),
        (
            "dormir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo dormo",
                "tu dorms",
                "ell dorm",
                "nosaltres dormim",
                "vosaltres dormiu",
                "ells dormen",
            ],
        ),
        (
            "tossir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo tusso",
                "tu tusses",
                "ell tus",
                "nosaltres tossim",
                "vosaltres tossiu",
                "ells tussen",
            ],
        ),
        (
            "collir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo culli",
                "tu cullis",
                "ell culli",
                "nosaltres collim",
                "vosaltres colliu",
                "ells cullin",
            ],
        ),
        (
            "dormir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo dormi",
                "tu dormis",
                "ell dormi",
                "nosaltres dormim",
                "vosaltres dormiu",
                "ells dormin",
            ],
        ),
        (
            "tossir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo tussi",
                "tu tussis",
                "ell tussi",
                "nosaltres tossim",
                "vosaltres tossiu",
                "ells tussin",
            ],
        ),
        (
            "dir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo dic",
                "tu dius",
                "ell diu",
                "nosaltres diem",
                "vosaltres dieu",
                "ells diuen",
            ],
        ),
        (
            "anar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo vaig",
                "tu vas",
                "ell va",
                "nosaltres anem",
                "vosaltres aneu",
                "ells van",
            ],
        ),
        (
            "poder",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo puc",
                "tu pots",
                "ell pot",
                "nosaltres podem",
                "vosaltres podeu",
                "ells poden",
            ],
        ),
        (
            "arribar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo arribo",
                "tu arribes",
                "ell arriba",
                "nosaltres arribem",
                "vosaltres arribeu",
                "ells arriben",
            ],
        ),
        (
            "trobar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo trobo",
                "tu trobes",
                "ell troba",
                "nosaltres trobem",
                "vosaltres trobeu",
                "ells troben",
            ],
        ),
        (
            "partir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo parteixo",
                "tu parteixes",
                "ell parteix",
                "nosaltres partim",
                "vosaltres partiu",
                "ells parteixen",
            ],
        ),
        (
            "morir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo moro",
                "tu mors",
                "ell mor",
                "nosaltres morim",
                "vosaltres moriu",
                "ells moren",
            ],
        ),
        (
            "eixir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo ixo",
                "tu ixes",
                "ell ix",
                "nosaltres eixim",
                "vosaltres eixiu",
                "ells ixen",
            ],
        ),
        (
            "jaure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo jec",
                "tu jeus",
                "ell jeu",
                "nosaltres jaiem",
                "vosaltres jaieu",
                "ells jeuen",
            ],
        ),
        (
            "jeure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo jec",
                "tu jeus",
                "ell jeu",
                "nosaltres jaiem",
                "vosaltres jaieu",
                "ells jeuen",
            ],
        ),
        (
            "pagar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo pago",
                "tu pagues",
                "ell paga",
                "nosaltres paguem",
                "vosaltres pagueu",
                "ells paguen",
            ],
        ),
        (
            "mentir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo menteixo",
                "tu menteixes",
                "ell menteix",
                "nosaltres mentim",
                "vosaltres mentiu",
                "ells menteixen",
            ],
        ),
        (
            "desmentir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo desmenteixo",
                "tu desmenteixes",
                "ell desmenteix",
                "nosaltres desmentim",
                "vosaltres desmentiu",
                "ells desmenteixen",
            ],
        ),
        (
            "nàixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo naixo",
                "tu naixes",
                "ell naix",
                "nosaltres naixem",
                "vosaltres naixeu",
                "ells naixen",
            ],
        ),
        (
            "néixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo naixo",
                "tu naixes",
                "ell naix",
                "nosaltres naixem",
                "vosaltres naixeu",
                "ells naixen",
            ],
        ),
        (
            "obrir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo obro",
                "tu obres",
                "ell obre",
                "nosaltres obrim",
                "vosaltres obriu",
                "ells obren",
            ],
        ),
        (
            "riure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo ric",
                "tu rius",
                "ell riu",
                "nosaltres riem",
                "vosaltres rieu",
                "ells riuen",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo pertanyo",
                "tu pertanys",
                "ell pertany",
                "nosaltres pertanyem",
                "vosaltres pertanyeu",
                "ells pertanyen",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo pertanyí",
                "tu pertanyeres",
                "ell pertanyé",
                "nosaltres pertanyérem",
                "vosaltres pertanyéreu",
                "ells pertanyeren",
            ],
        ),
        (
            "pertànyer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            [
                "jo pertanguí",
                "tu pertangueres",
                "ell pertangué",
                "nosaltres pertanguérem",
                "vosaltres pertanguéreu",
                "ells pertanguéren",
            ],
        ),
        (
            "fènyer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo fenyo",
                "tu fenys",
                "ell feny",
                "nosaltres fenyem",
                "vosaltres fenyeu",
                "ells fenyen",
            ],
        ),
        (
            "córrer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo corro",
                "tu corres",
                "ell corre",
                "nosaltres correm",
                "vosaltres correu",
                "ells corren",
            ],
        ),
        (
            "transcórrer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo transcorro",
                "tu transcorres",
                "ell transcorre",
                "nosaltres transcorrem",
                "vosaltres transcorreu",
                "ells transcorren",
            ],
        ),
        (
            "imprimir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            [
                "jo imprimeixo",
                "tu imprimeixes",
                "ell imprimeix",
                "nosaltres imprimim",
                "vosaltres imprimiu",
                "ells imprimeixen",
            ],
        ),
        (
            "imprimir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo imprimeixo",
                "tu imprimeixes",
                "ella imprimeix",
                "nosaltres imprimim",
                "vosaltres imprimiu",
                "elles imprimeixen",
            ],
        ),
        (
            "vindre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo vindo",
                "tu vinds",
                "ella vind",
                "nosaltres vindem",
                "vosaltres vindeu",
                "elles vinden",
            ],
        ),
        (
            "tòrcer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo torço",
                "tu torces",
                "ella torça",
                "nosaltres torcem",
                "vosaltres torceu",
                "elles torcen",
            ],
        ),
        (
            "caldre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "-",
                "-",
                "ella cal",
                "-",
                "-",
                "elles calen",
            ],
        ),
        (
            "obeir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo obeeixo",
                "tu obeeixes",
                "ella obeeix",
                "nosaltres obeïm",
                "vosaltres obeïu",
                "elles obeeixen",
            ],
        ),
        (
            "beneir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo beneeixo",
                "tu beneeixes",
                "ella beneeix",
                "nosaltres beneïm",
                "vosaltres beneïu",
                "elles beneeixen",
            ],
        ),
        (
            "coldre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo colc",
                "tu cols",
                "ella col",
                "nosaltres colem",
                "vosaltres coleu",
                "elles colen",
            ],
        ),
        (
            "dissoldre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo dissolc",
                "tu dissols",
                "ella dissol",
                "nosaltres dissolem",
                "vosaltres dissoleu",
                "elles dissolen",
            ],
        ),
        (
            "percudir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo percudeixo",
                "tu percudeixes",
                "ella percudeix",
                "nosaltres percudim",
                "vosaltres percudiu",
                "elles percudeixen",
            ],
        ),
        (
            "percudir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo percudo",
                "tu percuts",
                "ella percut",
                "nosaltres percudim",
                "vosaltres percudiu",
                "elles percuden",
            ],
        ),
        (
            "permetre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo permeto",
                "tu permets",
                "ella permet",
                "nosaltres permetem",
                "vosaltres permeteu",
                "elles permeten",
            ],
        ),
        (
            "desconèixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo desconec",
                "tu desconeixes",
                "ella desconeix",
                "nosaltres desconeixem",
                "vosaltres desconeixeu",
                "elles desconeixen",
            ],
        ),
        (
            "desconèixer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo desconeguí",
                "tu desconegueres",
                "ella desconegué",
                "nosaltres desconeguérem",
                "vosaltres desconeguéreu",
                "elles desconegueren",
            ],
        ),
        (
            "parèixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo parec",
                "tu pareixes",
                "ella pareix",
                "nosaltres pareixem",
                "vosaltres pareixeu",
                "elles pareixen",
            ],
        ),
        (
            "expendre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo expenc",
                "tu expens",
                "ella expèn",
                "nosaltres expenem",
                "vosaltres expeneu",
                "elles expenen",
            ],
        ),
        (
            "contendre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo contenc",
                "tu contens",
                "ella contén",
                "nosaltres contenem",
                "vosaltres conteneu",
                "elles contenen",
            ],
        ),
        (
            "atènyer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo atenyo",
                "tu atenys",
                "ella ateny",
                "nosaltres atenyem",
                "vosaltres atenyeu",
                "elles atenyen",
            ],
        ),
        (
            "descloure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo descloc",
                "tu desclous",
                "ella desclou",
                "nosaltres descloem",
                "vosaltres descloeu",
                "elles desclouen",
            ],
        ),
        (
            "descloure",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["desclòs", "desclosa", "desclosos", "descloses"],
        ),
        (
            "cloure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo cloc",
                "tu clous",
                "ella clou",
                "nosaltres cloem",
                "vosaltres cloeu",
                "elles clouen",
            ],
        ),
        (
            "abreviar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo abrevio",
                "tu abrevies",
                "ella abrevia",
                "nosaltres abreviem",
                "vosaltres abrevieu",
                "elles abrevien",
            ],
        ),
        (
            "abreviar",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo abreviï",
                "tu abreviïs",
                "ella abreviï",
                "nosaltres abreviem",
                "vosaltres abrevieu",
                "elles abreviïn",
            ],
        ),
        (
            "abreviar",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["abrevia", "abreviï", "abreviem", "abrevieu", "abreviïn"],
        ),
        (
            "compondre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo componc",
                "tu compons",
                "ella compon",
                "nosaltres componem",
                "vosaltres componeu",
                "elles componen",
            ],
        ),
        (
            "contendre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo contenc",
                "tu contens",
                "ella contén",
                "nosaltres contenem",
                "vosaltres conteneu",
                "elles contenen",
            ],
        ),
        (
            "complir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo compleixo",
                "tu compleixes",
                "ella compleix",
                "nosaltres complim",
                "vosaltres compliu",
                "elles compleixen",
            ],
        ),
        (
            "establir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo estableixo",
                "tu estableixes",
                "ella estableix",
                "nosaltres establim",
                "vosaltres establiu",
                "elles estableixen",
            ],
        ),
        (
            "complaure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo complac",
                "tu complaus",
                "ella complau",
                "nosaltres complaem",
                "vosaltres complaeu",
                "elles complauen",
            ],
        ),
        (
            "desplaure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo desplac",
                "tu desplaus",
                "ella desplau",
                "nosaltres desplaem",
                "vosaltres desplaeu",
                "elles desplauen",
            ],
        ),
        (
            "complaure",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["complagut", "complaguda", "complaguts", "complagudes"],
        ),
        (
            "acréixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo acreixo",
                "tu acreixes",
                "ella acreix",
                "nosaltres acreixem",
                "vosaltres acreixeu",
                "elles acreixen",
            ],
        ),
        (
            "acréixer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo acreixí",
                "tu acreixeres",
                "ella acreixé",
                "nosaltres acreixérem",
                "vosaltres acreixéreu",
                "elles acreixeren",
            ],
        ),
        (
            "acréixer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo acresquí",
                "tu acresqueres",
                "ella acresqué",
                "nosaltres acresquérem",
                "vosaltres acresquéreu",
                "elles acresqueren",
            ],
        ),
        (
            "acréixer",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo acreixés",
                "tu acreixessis",
                "ella acreixés",
                "nosaltres acreixéssim",
                "vosaltres acreixéssiu",
                "elles acreixessin",
            ],
        ),
        (
            "acréixer",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo acresqués",
                "tu acresquessis",
                "ella acresqués",
                "nosaltres acresquéssim",
                "vosaltres acresquéssiu",
                "elles acresquessin",
            ],
        ),
        (
            "merèixer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo mereixo",
                "tu mereixes",
                "ella mereix",
                "nosaltres mereixem",
                "vosaltres mereixeu",
                "elles mereixen",
            ],
        ),
        (
            "confondre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo confonc",
                "tu confons",
                "ella confon",
                "nosaltres confonem",
                "vosaltres confoneu",
                "elles confonen",
            ],
        ),
        (
            "consternar",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo consterno",
                "tu consternes",
                "ella consterna",
                "nosaltres consternem",
                "vosaltres consterneu",
                "elles consternen",
            ],
        ),
        (
            "contradir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo contradic",
                "tu contradius",
                "ella contradiu",
                "nosaltres contradiem",
                "vosaltres contradieu",
                "elles contradiuen",
            ],
        ),
        (
            "contradir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["contradit", "contradida", "contradits", "contradides"],
        ),
        (
            "dir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["dit", "dita", "dits", "dites"],
        ),
        (
            "creure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo crec",
                "tu creus",
                "ella creu",
                "nosaltres creiem",
                "vosaltres creieu",
                "elles creuen",
            ],
        ),
        (
            "creure",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo creuré",
                "tu creuràs",
                "ella creurà",
                "nosaltres creurem",
                "vosaltres creureu",
                "elles creuran",
            ],
        ),
        (
            "dur",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo duc",
                "tu duus",
                "ella duu",
                "nosaltres duem",
                "vosaltres dueu",
                "elles duen",
            ],
        ),
        (
            "dur",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo duc",
                "tu dus",
                "ella du",
                "nosaltres duem",
                "vosaltres dueu",
                "elles duen",
            ],
        ),
        (
            "dur",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["duu", "dugui", "duguem", "dueu", "duguin"],
        ),
        (
            "dur",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["du", "dugui", "duguem", "dueu", "duguin"],
        ),
        (
            "descosir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo descuso",
                "tu descuses",
                "ella descús",
                "nosaltres descosim",
                "vosaltres descosiu",
                "elles descusen",
            ],
        ),
        (
            "descosir",
            "subjuntiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo descusi",
                "tu descusis",
                "ella descusi",
                "nosaltres descosim",
                "vosaltres descosiu",
                "elles descusin",
            ],
        ),
        (
            "descosir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["descús", "descusi", "descosim", "descosiu", "descusin"],
        ),
        (
            "valer",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo valc",
                "tu vals",
                "ella val",
                "nosaltres valem",
                "vosaltres valeu",
                "elles valen",
            ],
        ),
        (
            "valer",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo valguí",
                "tu valgueres",
                "ella valgué",
                "nosaltres valguérem",
                "vosaltres valguéreu",
                "elles valgueren",
            ],
        ),
        (
            "doldre",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo dolc",
                "tu dols",
                "ella dol",
                "nosaltres dolem",
                "vosaltres doleu",
                "elles dolen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo entrelluo",
                "tu entrelluus",
                "ella entrelluu",
                "nosaltres entrelluïm",
                "vosaltres entrelluïu",
                "elles entrelluen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo entrelluo",
                "tu entrelluus",
                "ella entrellú",
                "nosaltres entrelluïm",
                "vosaltres entrelluïu",
                "elles entrelluen",
            ],
        ),
        (
            "entrelluir",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo entrelluïa",
                "tu entrelluïes",
                "ella entrelluïa",
                "nosaltres entrelluíem",
                "vosaltres entrelluíeu",
                "elles entrelluïen",
            ],
        ),
        (
            "entrelluir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["entrelluu", "entrelluï", "entrelluïm", "entrelluïu", "entrelluïn"],
        ),
        (
            "entrelluir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["entrellú", "entrelluï", "entrelluïm", "entrelluïu", "entrelluïn"],
        ),
        (
            "esglaiar",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo esglaio",
                "tu esglaies",
                "ella esglaia",
                "nosaltres esglaiem",
                "vosaltres esglaieu",
                "elles esglaien",
            ],
        ),
        (
            "esglaiar",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo esglaí",
                "tu esglaiares",
                "ella esglaià",
                "nosaltres esglaiàrem",
                "vosaltres esglaiàreu",
                "elles esglaiaren",
            ],
        ),
        (
            "esglaiar",
            "subjuntiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo esglaï",
                "tu esglaïs",
                "ella esglaï",
                "nosaltres esglaiem",
                "vosaltres esglaieu",
                "elles esglaïn",
            ],
        ),
        (
            "esglaiar",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["esglaia", "esglaï", "esglaiem", "esglaieu", "esglaïn"],
        ),
        (
            "esglaiar",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["esglaiat", "esglaiada", "esglaiats", "esglaiades"],
        ),
        (
            "evacuar",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo evacuo",
                "tu evacues",
                "ella evacua",
                "nosaltres evacuem",
                "vosaltres evacueu",
                "elles evacuen",
            ],
        ),
        (
            "evacuar",
            "subjuntiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo evacuï",
                "tu evacuïs",
                "ella evacuï",
                "nosaltres evacuem",
                "vosaltres evacueu",
                "elles evacuïn",
            ],
        ),
        (
            "evacuar",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["evacua", "evacuï", "evacuem", "evacueu", "evacuïn"],
        ),
        (
            "tondre",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo tonc",
                "tu tons",
                "ella ton",
                "nosaltres tonem",
                "vosaltres toneu",
                "elles tonen",
            ],
        ),
        (
            "fugir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo fujo",
                "tu fuges",
                "ella fuig",
                "nosaltres fugim",
                "vosaltres fugiu",
                "elles fugen",
            ],
        ),
        (
            "fugir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["fuig", "fugi", "fugim", "fugiu", "fugin"],
        ),
        (
            "imprimir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo imprimeixo",
                "tu imprimeixes",
                "ella imprimeix",
                "nosaltres imprimim",
                "vosaltres imprimiu",
                "elles imprimeixen",
            ],
        ),
        (
            "imprimir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["imprès", "impresa", "impresos", "impreses"],
        ),
        (
            "ajeure",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo ajec",
                "tu ajeus",
                "ella ajeu",
                "nosaltres ajaiem",
                "vosaltres ajaieu",
                "elles ajeuen",
            ],
        ),
        (
            "moldre",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo molc",
                "tu mols",
                "ella mol",
                "nosaltres molem",
                "vosaltres moleu",
                "elles molen",
            ],
        ),
        (
            "malvendre",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo malvenc",
                "tu malvens",
                "ella malvèn",
                "nosaltres malvenem",
                "vosaltres malveneu",
                "elles malvenen",
            ],
        ),
        (
            "malvendre",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo malvenia",
                "tu malvenies",
                "ella malvenia",
                "nosaltres malveníem",
                "vosaltres malveníeu",
                "elles malvenien",
            ],
        ),
        (
            "malvendre",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo malvenguí",
                "tu malvengueres",
                "ella malvengué",
                "nosaltres malvenguérem",
                "vosaltres malvenguéreu",
                "elles malvengueren",
            ],
        ),
        (
            "malvendre",
            "subjuntiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo malvengui",
                "tu malvenguis",
                "ella malvengui",
                "nosaltres malvenguem",
                "vosaltres malvengueu",
                "elles malvenguin",
            ],
        ),
        (
            "malvendre",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo malvengués",
                "tu malvenguessis",
                "ella malvengués",
                "nosaltres malvenguéssim",
                "vosaltres malvenguéssiu",
                "elles malvenguessin",
            ],
        ),
        (
            "omplir",
            "indicatiu",
            "present",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            [
                "jo omplo",
                "tu omples",
                "ella omple",
                "nosaltres omplim",
                "vosaltres ompliu",
                "elles omplen",
            ],
        ),
        (
            "omplir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["omplert", "omplerta", "omplerts", "omplertes"],
        ),
        (
            "omplir",
            "particip",
            "particip",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            ["omplit", "omplida", "omplits", "omplides"],
        ),
        (
            "omplir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.SecondOnly,
            Gender.m,
            ["omple", "ompli", "omplim", "ompliu", "omplin"],
        ),
        (
            "moure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo moc",
                "tu mous",
                "ella mou",
                "nosaltres movem",
                "vosaltres moveu",
                "elles mouen",
            ],
        ),
        (
            "moure",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo moguí",
                "tu mogueres",
                "ella mogué",
                "nosaltres moguérem",
                "vosaltres moguéreu",
                "elles mogueren",
            ],
        ),
        (
            "moure",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo movia",
                "tu movies",
                "ella movia",
                "nosaltres movíem",
                "vosaltres movíeu",
                "elles movien",
            ],
        ),
        (
            "absoldre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo absolc",
                "tu absols",
                "ella absol",
                "nosaltres absolem",
                "vosaltres absoleu",
                "elles absolen",
            ],
        ),
        (
            "absoldre",
            "particip",
            "particip",
            AlternatesBehavior.SecondOnly,
            Gender.f,
            ["absolt", "absolta", "absolts", "absoltes"],
        ),
        (
            "obrir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo obro",
                "tu obres",
                "ella obre",
                "nosaltres obrim",
                "vosaltres obriu",
                "elles obren",
            ],
        ),
        (
            "obrir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["obert", "oberta", "oberts", "obertes"],
        ),
        (
            "prevenir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previnc",
                "tu prevens",
                "ella prevé",
                "nosaltres prevenim",
                "vosaltres preveniu",
                "elles prevenen",
            ],
        ),
        (
            "prevenir",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previnguí",
                "tu previngueres",
                "ella previngué",
                "nosaltres previnguérem",
                "vosaltres previnguéreu",
                "elles previngueren",
            ],
        ),
        (
            "prevenir",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo prevenia",
                "tu prevenies",
                "ella prevenia",
                "nosaltres preveníem",
                "vosaltres preveníeu",
                "elles prevenien",
            ],
        ),
        (
            "prevenir",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previndré",
                "tu previndràs",
                "ella previndrà",
                "nosaltres previndrem",
                "vosaltres previndreu",
                "elles previndran",
            ],
        ),
        (
            "prevenir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previngui",
                "tu previnguis",
                "ella previngui",
                "nosaltres previnguem",
                "vosaltres previngueu",
                "elles previnguin",
            ],
        ),
        (
            "prevenir",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previngués",
                "tu previnguessis",
                "ella previngués",
                "nosaltres previnguéssim",
                "vosaltres previnguéssiu",
                "elles previnguessin",
            ],
        ),
        (
            "prevenir",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo previndria",
                "tu previndries",
                "ella previndria",
                "nosaltres previndríem",
                "vosaltres previndríeu",
                "elles previndrien",
            ],
        ),
        (
            "prevenir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["previngut", "previnguda", "previnguts", "previngudes"],
        ),
        (
            "prevenir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["prevén", "previngui", "previnguem", "preveniu", "previnguin"],
        ),
        (
            "sortir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo surto",
                "tu surts",
                "ella surt",
                "nosaltres sortim",
                "vosaltres sortiu",
                "elles surten",
            ],
        ),
        (
            "sortir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo surti",
                "tu surtis",
                "ella surti",
                "nosaltres sortim",
                "vosaltres sortiu",
                "elles surtin",
            ],
        ),
        (
            "sortir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["sortit", "sortida", "sortits", "sortides"],
        ),
        (
            "sortir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["surt", "surti", "sortim", "sortiu", "surtin"],
        ),
        (
            "témer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo temo",
                "tu tems",
                "ella tem",
                "nosaltres temem",
                "vosaltres temeu",
                "elles temen",
            ],
        ),
        (
            "témer",
            "infinitiu",
            "infinitiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["témer"],
        ),
        (
            "témer",
            "gerundi",
            "gerundi",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["tement"],
        ),
        (
            "témer",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["temut", "temuda", "temuts", "temudes"],
        ),
        (
            "retraure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retrec",
                "tu retreus",
                "ella retreu",
                "nosaltres retraiem",
                "vosaltres retraieu",
                "elles retreuen",
            ],
        ),
        (
            "retraure",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retreia",
                "tu retreies",
                "ella retreia",
                "nosaltres retrèiem",
                "vosaltres retrèieu",
                "elles retreien",
            ],
        ),
        (
            "retraure",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retraguí",
                "tu retragueres",
                "ella retragué",
                "nosaltres retraguérem",
                "vosaltres retraguéreu",
                "elles retragueren",
            ],
        ),
        (
            "retraure",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retraué",
                "tu retrauàs",
                "ella retrauà",
                "nosaltres retrauem",
                "vosaltres retraueu",
                "elles retrauan",
            ],
        ),
        (
            "retraure",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retrauia",
                "tu retrauies",
                "ella retrauia",
                "nosaltres retrauíem",
                "vosaltres retrauíeu",
                "elles retrauien",
            ],
        ),
        (
            "retraure",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo retregui",
                "tu retreguis",
                "ella retregui",
                "nosaltres retreguem",
                "vosaltres retregueu",
                "elles retreguin",
            ],
        ),
        (
            "premorir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo premoro",
                "tu premors",
                "ella premor",
                "nosaltres premorim",
                "vosaltres premoriu",
                "elles premoren",
            ],
        ),
        (
            "conviure",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo convisc",
                "tu convius",
                "ella conviu",
                "nosaltres convivim",
                "vosaltres conviviu",
                "elles conviuen",
            ],
        ),
        (
            "conviure",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo convivia",
                "tu convivies",
                "ella convivia",
                "nosaltres convivíem",
                "vosaltres convivíeu",
                "elles convivien",
            ],
        ),
        (
            "conviure",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo convisquí",
                "tu convisqueres",
                "ella convisqué",
                "nosaltres convisquérem",
                "vosaltres convisquéreu",
                "elles convisqueren",
            ],
        ),
        (
            "vèncer",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo venço",
                "tu vences",
                "ella venç",
                "nosaltres vencem",
                "vosaltres venceu",
                "elles vencen",
            ],
        ),
        (
            "vèncer",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["venç", "venci", "vencem", "venceu", "vencin"],
        ),
        (
            "vèncer",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["vençut", "vençuda", "vençuts", "vençudes"],
        ),
        (
            "vèncer",
            "infinitiu",
            "infinitiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["vèncer"],
        ),
        (
            "vèncer",
            "gerundi",
            "gerundi",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["vencent"],
        ),
        (
            "cerndre",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo cerno",
                "tu cerns",
                "ella cern",
                "nosaltres cernem",
                "vosaltres cerneu",
                "elles cernen",
            ],
        ),
        (
            "oir",
            "indicatiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oeixo",
                "tu oeixes",
                "ella oeix",
                "nosaltres oïm",
                "vosaltres oïu",
                "elles oeixen",
            ],
        ),
        (
            "oir",
            "indicatiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oïa",
                "tu oïes",
                "ella oïa",
                "nosaltres oíem",
                "vosaltres oíeu",
                "elles oïen",
            ],
        ),
        (
            "oir",
            "indicatiu",
            "passat-simple",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oí",
                "tu oïres",
                "ella oí",
                "nosaltres oírem",
                "vosaltres oíreu",
                "elles oïren",
            ],
        ),
        (
            "oir",
            "indicatiu",
            "futur",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oiré",
                "tu oiràs",
                "ella oirà",
                "nosaltres oirem",
                "vosaltres oireu",
                "elles oiran",
            ],
        ),
        (
            "oir",
            "subjuntiu",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oeixi",
                "tu oeixis",
                "ella oeixi",
                "nosaltres oïm",
                "vosaltres oïu",
                "elles oeixin",
            ],
        ),
        (
            "oir",
            "subjuntiu",
            "imperfet",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oís",
                "tu oïssis",
                "ella oís",
                "nosaltres oíssim",
                "vosaltres oíssiu",
                "elles oïssin",
            ],
        ),
        (
            "oir",
            "condicional",
            "present",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            [
                "jo oiria",
                "tu oiries",
                "ella oiria",
                "nosaltres oiríem",
                "vosaltres oiríeu",
                "elles oirien",
            ],
        ),
        (
            "oir",
            "particip",
            "particip",
            AlternatesBehavior.FirstOnly,
            Gender.f,
            ["oït", "oïda", "oïts", "oïdes"],
        ),
        (
            "oir",
            "infinitiu",
            "infinitiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["oir"],
        ),
        (
            "oir",
            "gerundi",
            "gerundi",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["oint"],
        ),
        (
            "oir",
            "imperatiu",
            "imperatiu-present",
            AlternatesBehavior.FirstOnly,
            Gender.m,
            ["oeix", "oeixi", "oïm", "oïu", "oeixin"],
        ),
    ],
)
def test_inflector_ca_conjugate_mood_tense(
    cg, infinitive, mood, tense, alternates_behavior, gender, expected_result
):
    co = cg._get_conj_obs(infinitive)
    print("template:" + co.template.name)
    assert (
        cg.conjugate_mood_tense(infinitive, mood, tense, alternates_behavior, gender)
        == expected_result
    )


def test_inflector_ca_conjugate_simple_past(cg):
    assert cg.conjugate_mood_tense(
        "pertànyer",
        "indicatiu",
        "passat-simple",
        AlternatesBehavior.FirstOnly,
        Gender.m,
    ) == [
        "jo pertanyí",
        "tu pertanyeres",
        "ell pertanyé",
        "nosaltres pertanyérem",
        "vosaltres pertanyéreu",
        "ells pertanyeren",
    ]


def test_inflector_ca_conjugate_simple_past_alternate(cg):
    assert cg.conjugate_mood_tense(
        "pertànyer",
        "indicatiu",
        "passat-simple",
        AlternatesBehavior.SecondOnly,
        Gender.m,
    ) == [
        "jo pertanguí",
        "tu pertangueres",
        "ell pertangué",
        "nosaltres pertanguérem",
        "vosaltres pertanguéreu",
        "ells pertanguéren",
    ]


def test_inflector_ca_get_conj_obs(cg):
    co = cg._get_conj_obs("parlar")
    assert co.verb.infinitive == "parlar"
    assert co.verb_stem == "parl"
    assert co.template.name == "cant:ar"


def test_inflector_ca_get_conj_obs_2(cg):
    co = cg._get_conj_obs("abandonar")
    assert co.verb.infinitive == "abandonar"
    assert co.verb_stem == "abandon"
    assert co.template.name == "cant:ar"


def test_inflector_ca_get_conj_obs_3(cg):
    co = cg._get_conj_obs("pertànyer")
    assert co.verb.infinitive == "pertànyer"
    assert co.verb_stem == "pertàny"
    assert co.template.name == "pertàny:er"
    assert co.template.modify_stem == "strip-accents"


def test_inflector_ca_get_verb_stem_from_template_name(cg):
    verb_stem = cg._inflector._get_verb_stem_from_template_name("parlar", "cant:ar")
    assert verb_stem == "parl"


def test_inflector_ca_get_verb_stem_from_template_name_2(cg):
    verb_stem = cg._inflector._get_verb_stem_from_template_name("abandonar", "cant:ar")
    assert verb_stem == "abandon"


def test_inflector_ca_conjugate_simple_mood_tense(cg):
    mood = "indicatiu"
    tense = "present"
    verb_stem = "parl"
    tense_elem = etree.fromstring(
        """<present>
            <p><i>o</i></p>
            <p><i>es</i></p>
            <p><i>a</i></p>
            <p><i>em</i></p>
            <p><i>eu</i></p>
            <p><i>en</i></p>
        </present>""",
        parser=None,
    )
    tense_template = TenseTemplate(Lang.ca, mood, tense_elem)
    out = cg._conjugate_simple_mood_tense(verb_stem, mood, tense, tense_template)
    assert len(out) == 6
    assert out == [
        "jo parlo",
        "tu parles",
        "ell parla",
        "nosaltres parlem",
        "vosaltres parleu",
        "ells parlen",
    ]


@pytest.mark.parametrize(
    "person,gender,is_reflexive,expected_result",
    [
        (Person.FirstPersonSingular, Gender.m, False, "jo"),
        (Person.FirstPersonSingular, Gender.m, True, "jo me"),
        (Person.SecondPersonSingular, Gender.m, False, "tu"),
        (Person.SecondPersonSingular, Gender.m, True, "tu te"),
        (Person.ThirdPersonSingular, Gender.m, False, "ell"),
        (Person.ThirdPersonSingular, Gender.m, True, "ell se"),
        (Person.ThirdPersonSingular, Gender.f, False, "ella"),
        (Person.ThirdPersonSingular, Gender.f, True, "ella se"),
        (Person.FirstPersonPlural, Gender.m, False, "nosaltres"),
        (Person.FirstPersonPlural, Gender.m, True, "nosaltres nos"),
        (Person.SecondPersonPlural, Gender.m, False, "vosaltres"),
        (Person.SecondPersonPlural, Gender.m, True, "vosaltres os"),
        (Person.ThirdPersonPlural, Gender.m, False, "ells"),
        (Person.ThirdPersonPlural, Gender.m, True, "ells se"),
        (Person.ThirdPersonPlural, Gender.f, False, "elles"),
        (Person.ThirdPersonPlural, Gender.f, True, "elles se"),
    ],
)
def test_inflector_ca_get_default_pronoun(
    cg, person, gender, is_reflexive, expected_result
):
    assert (
        cg._inflector.get_default_pronoun(person, gender, is_reflexive=is_reflexive)
        == expected_result
    )


def test_inflector_conjugate(cg):
    assert cg.conjugate("ser") == {
        "verb": {
            "infinitive": "ser",
            "predicted": False,
            "pred_score": 1.0,
            "template": "és:ser",
            "translation_en": "be",
            "stem": "",
        },
        "moods": {
            "indicatiu": {
                "present": [
                    "jo sóc",
                    "tu ets",
                    "ell és",
                    "nosaltres som",
                    "vosaltres sou",
                    "ells són",
                ],
                "imperfet": [
                    "jo era",
                    "tu eres",
                    "ell era",
                    "nosaltres érem",
                    "vosaltres éreu",
                    "ells eren",
                ],
                "passat-simple": [
                    "jo fui",
                    "tu fores",
                    "ell fou",
                    "nosaltres fórem",
                    "vosaltres fóreu",
                    "ells foren",
                ],
                "futur": [
                    "jo seré",
                    "tu seràs",
                    "ell serà",
                    "nosaltres serem",
                    "vosaltres sereu",
                    "ells seran",
                ],
            },
            "subjuntiu": {
                "present": [
                    "jo sigui",
                    "tu siguis",
                    "ell sigui",
                    "nosaltres siguem",
                    "vosaltres sigueu",
                    "ells siguin",
                ],
                "imperfet": [
                    "jo fos",
                    "tu fossis",
                    "ell fos",
                    "nosaltres fóssim",
                    "vosaltres fóssiu",
                    "ells fossin",
                ],
            },
            "imperatiu": {
                "imperatiu-present": ["sigues", "sigui", "siguem", "sigueu", "siguin"]
            },
            "condicional": {
                "present": [
                    "jo seria",
                    "tu series",
                    "ell seria",
                    "nosaltres seríem",
                    "vosaltres seríeu",
                    "ells serien",
                ]
            },
            "infinitiu": {"infinitiu-present": ["ser"]},
            "gerundi": {"gerundi": ["sent"]},
            "particip": {"particip": ["estat", "estada", "estats", "estades"]},
        },
    }


def test_inflector_conjugate_with_alternates(cg):
    assert cg.conjugate("ser", include_alternates=True) == {
        "verb": {
            "infinitive": "ser",
            "predicted": False,
            "pred_score": 1.0,
            "template": "és:ser",
            "translation_en": "be",
            "stem": "",
        },
        "moods": {
            "indicatiu": {
                "present": [
                    ["jo sóc"],
                    ["tu ets"],
                    ["ell és"],
                    ["nosaltres som"],
                    ["vosaltres sou"],
                    ["ells són"],
                ],
                "imperfet": [
                    ["jo era"],
                    ["tu eres"],
                    ["ell era"],
                    ["nosaltres érem"],
                    ["vosaltres éreu"],
                    ["ells eren"],
                ],
                "passat-simple": [
                    ["jo fui"],
                    ["tu fores"],
                    ["ell fou"],
                    ["nosaltres fórem"],
                    ["vosaltres fóreu"],
                    ["ells foren"],
                ],
                "futur": [
                    ["jo seré"],
                    ["tu seràs"],
                    ["ell serà"],
                    ["nosaltres serem"],
                    ["vosaltres sereu"],
                    ["ells seran"],
                ],
            },
            "subjuntiu": {
                "present": [
                    ["jo sigui"],
                    ["tu siguis"],
                    ["ell sigui"],
                    ["nosaltres siguem"],
                    ["vosaltres sigueu"],
                    ["ells siguin"],
                ],
                "imperfet": [
                    ["jo fos"],
                    ["tu fossis"],
                    ["ell fos"],
                    ["nosaltres fóssim"],
                    ["vosaltres fóssiu"],
                    ["ells fossin"],
                ],
            },
            "imperatiu": {
                "imperatiu-present": [
                    ["sigues"],
                    ["sigui"],
                    ["siguem"],
                    ["sigueu"],
                    ["siguin"],
                ]
            },
            "condicional": {
                "present": [
                    ["jo seria", "jo fora"],
                    ["tu series", "tu fores"],
                    ["ell seria", "ell fora"],
                    ["nosaltres seríem", "nosaltres fórem"],
                    ["vosaltres seríeu", "vosaltres fóreu"],
                    ["ells serien", "ells foren"],
                ]
            },
            "infinitiu": {"infinitiu-present": [["ser", "ésser"]]},
            "gerundi": {"gerundi": [["sent", "essent"]]},
            "particip": {
                "particip": [
                    ["estat", "sigut"],
                    ["estada", "siguda"],
                    ["estats", "siguts"],
                    ["estades", "sigudes"],
                ]
            },
        },
    }


def test_inflector_conjugate_with_alternates_noconjpronouns(cg):
    assert cg.conjugate("ser", include_alternates=True, conjugate_pronouns=False) == {
        "verb": {
            "infinitive": "ser",
            "predicted": False,
            "pred_score": 1.0,
            "template": "és:ser",
            "translation_en": "be",
            "stem": "",
        },
        "moods": {
            "indicatiu": {
                "present": [
                    ["sóc"],
                    ["ets"],
                    ["és"],
                    ["som"],
                    ["sou"],
                    ["són"],
                ],
                "imperfet": [
                    ["era"],
                    ["eres"],
                    ["era"],
                    ["érem"],
                    ["éreu"],
                    ["eren"],
                ],
                "passat-simple": [
                    ["fui"],
                    ["fores"],
                    ["fou"],
                    ["fórem"],
                    ["fóreu"],
                    ["foren"],
                ],
                "futur": [
                    ["seré"],
                    ["seràs"],
                    ["serà"],
                    ["serem"],
                    ["sereu"],
                    ["seran"],
                ],
            },
            "subjuntiu": {
                "present": [
                    ["sigui"],
                    ["siguis"],
                    ["sigui"],
                    ["siguem"],
                    ["sigueu"],
                    ["siguin"],
                ],
                "imperfet": [
                    ["fos"],
                    ["fossis"],
                    ["fos"],
                    ["fóssim"],
                    ["fóssiu"],
                    ["fossin"],
                ],
            },
            "imperatiu": {
                "imperatiu-present": [
                    ["sigues"],
                    ["sigui"],
                    ["siguem"],
                    ["sigueu"],
                    ["siguin"],
                ]
            },
            "condicional": {
                "present": [
                    ["seria", "fora"],
                    ["series", "fores"],
                    ["seria", "fora"],
                    ["seríem", "fórem"],
                    ["seríeu", "fóreu"],
                    ["serien", "foren"],
                ]
            },
            "infinitiu": {"infinitiu-present": [["ser", "ésser"]]},
            "gerundi": {"gerundi": [["sent", "essent"]]},
            "particip": {
                "particip": [
                    ["estat", "sigut"],
                    ["estada", "siguda"],
                    ["estats", "siguts"],
                    ["estades", "sigudes"],
                ]
            },
        },
    }



import sys

if sys.version_info >= (3, 11):
    from enum import StrEnum
else:
    from backports.strenum import StrEnum


class MoodEn(StrEnum):
    Conditional = "conditional"
    Gerund = "gerund"
    Imperative = "imperative"
    Indicative = "indicative"
    Infinitive = "infinitive"
    Participle = "participle"
    Subjunctive = "subjunctive"


class MoodFr(StrEnum):
    Conditionnel = "conditionnel"
    Imperatif = "imperatif"
    Indicatif = "indicatif"
    Infinitif = "infinitif"
    Participe = "participe"
    Subjonctif = "subjonctif"


class MoodEs(StrEnum):
    Condicional = "condicional"
    Gerundio = "gerundio"
    Imperativo = "imperativo"
    Indicativo = "indicativo"
    Infinitivo = "infinitivo"
    Participo = "participo"
    Subjuntivo = "subjuntivo"


class MoodIt(StrEnum):
    Condizionale = "condizionale"
    Congiuntivo = "congiuntivo"
    Imperativo = "imperativo"
    Indicativo = "indicativo"
    Infinito = "infinito"
    Participio = "participio"


class MoodCa(StrEnum):
    Condicional = "condicional"
    Gerundi = "gerundi"
    Imperatiu = "imperatiu"
    Indicatiu = "indicatiu"
    Infinitiu = "infinitiu"
    Participi = "participi"
    Subjuntiu = "subjuntiu"


class MoodRo(StrEnum):
    Condițional = "condițional"
    Conjunctiv = "conjunctiv"
    Gerunziu = "gerunziu"
    Imperativ = "imperativ"
    Indicativ = "indicativ"
    Infinitiv = "infinitiv"
    Participiu = "participiu"
    Subjunctiv = "subjunctiv"


class MoodPt(StrEnum):
    Condicional = "condicional"
    Gerúndio = "gerúndio"
    Imperativo = "imperativo"
    Indicativo = "indicativo"
    Infinitivo = "infinitivo"
    Particípio = "particípio"
    Subjuntivo = "subjuntivo"


class Mood:
    ca = MoodCa
    en = MoodEn
    es = MoodEs
    fr = MoodFr
    it = MoodIt
    pt = MoodPt
    ro = MoodRo



from typing import Dict, List, Tuple

from verbecc.src.conjugator.conjugation_object import ConjugationObjects
from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.person import Person
from verbecc.src.defs.types.lang_code import LangCodeISO639_1
from verbecc.src.defs.types.mood import MoodRo as Mood
from verbecc.src.defs.types.tense import TenseRo as Tense
from verbecc.src.defs.types.lang_specific_options import (
    LangSpecificOptions,
)
from verbecc.src.inflectors.inflector import Inflector


class InflectorRo(Inflector):
    @property
    def lang(self) -> LangCodeISO639_1:
        return LangCodeISO639_1.ro

    def __init__(self) -> None:
        super(InflectorRo, self).__init__()

    def _add_subjunctive_relative_pronoun(self, s: str, tense: Tense) -> str:
        tokens = s.split(" ")
        if tense == Tense.Prezent:
            tokens.insert(1, "să")
        elif tense == Tense.Perfect:
            tokens.insert(1, "să fi")
        return " ".join(tokens)

    def _add_adverb_if_applicable(self, s: str, mood: Mood, tense: Tense) -> str:
        if mood == Mood.Imperativ and tense == Tense.Negativ:
            return "nu " + s
        return s

    """TODO: There are two types of reflexive verbs in Romanian: 
    preceded by the reflexive pronouns “se” (in the accusative) and “și” (in the dative).
    """

    def get_default_pronoun(
        self,
        person: Person,
        gender: Gender = Gender.m,
        is_reflexive: bool = False,
        lang_specific_options: LangSpecificOptions = None,
    ) -> str:
        ret = ""
        if person == Person.FirstPersonSingular:
            ret = "eu"
            if is_reflexive:
                ret += " mă"
        elif person == Person.SecondPersonSingular:
            ret = "tu"
            if is_reflexive:
                ret += " te"
        elif person == Person.ThirdPersonSingular:
            ret = "el"
            if gender == Gender.f:
                ret = "ea"
            if is_reflexive:
                ret += " se"
        elif person == Person.FirstPersonPlural:
            ret = "noi"
            if is_reflexive:
                ret += " ne"
        elif person == Person.SecondPersonPlural:
            ret = "voi"
            if is_reflexive:
                ret += " vă"
        elif person == Person.ThirdPersonPlural:
            ret = "ei"
            if gender == Gender.f:
                ret = "ele"
            if is_reflexive:
                ret += " se"
        return ret

    def _get_tenses_conjugated_without_pronouns(self) -> List[Tense]:
        return [
            Tense.Participiu,
            Tense.Afirmativ,
            Tense.Imperativ,
            Tense.Negativ,
            Tense.Gerunziu,
        ]

    def _get_auxilary_verb(
        self, co: ConjugationObjects, mood: Mood, tense: Tense
    ) -> str:
        if tense in (Tense.Viitor1, Tense.Viitor2):
            return "voi"
        elif tense == Tense.Viitor1Popular:
            return co.verb.infinitive
        return "avea"

    def _get_infinitive_mood(self) -> Mood:
        return Mood.Infinitiv

    def _get_indicative_mood(self) -> Mood:
        return Mood.Indicativ

    def _get_subjunctive_mood(self) -> Mood:
        return Mood.Conjunctiv

    def _get_conditional_mood(self) -> Mood:
        return Mood.Condițional

    def _get_participle_mood(self) -> Mood:
        return Mood.Participiu

    def _get_participle_tense(self) -> Tense:
        return Tense.Participiu

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[Mood, Dict[Tense, Tuple[Mood, Tense]]]:
        # TODO: those last three don't actually use an auxiliary verb, refactor to make aux verb optional
        return {
            Mood.Indicativ: {
                Tense.PerfectCompus: (Mood.Indicativ, Tense.Prezent),
                Tense.Viitor1: (Mood.Indicativ, Tense.Prezent),
                Tense.Viitor2: (Mood.Indicativ, Tense.Prezent),
                Tense.Viitor1Popular: (Mood.Conjunctiv, Tense.Prezent),
                Tense.Viitor2Popular: (Mood.Indicativ, Tense.Prezent),
            },
            Mood.Conjunctiv: {Tense.Perfect: (Mood.Indicativ, Tense.Prezent)},
            Mood.Condițional: {
                Tense.Prezent: (Mood.Indicativ, Tense.Prezent),
                Tense.Perfect: (Mood.Indicativ, Tense.Prezent),
            },
        }

    def _auxilary_verb_uses_alternate_conjugation(self, tense: Tense) -> bool:
        return tense.startswith("viitor")

    def _compound_primary_verb_conjugation_uses_infinitive(
        self, mood: Mood, tense: Tense
    ) -> bool:
        if mood == Mood.Indicativ and tense == Tense.Viitor1:
            return True
        elif mood == Mood.Condițional and tense == Tense.Prezent:
            return True
        return False

    def _modify_aux_verb_conj_if_applicable(
        self, aux_conj: List[str], mood: Mood, tense: Tense
    ) -> List[str]:
        """E.g. for Romanian conditional present 'eu aş avea, tu ai avea, el ar avea, ...'
        and also the Romanian conditional present e.g. 'eu	aş fi avut, tu ai fi avut, ...'
        although the ' fi' is added by _add_compound_aux_verb_suffix_if_applicable

        Normally Romanian aux_conj would be the indicativ prezent tense of avea i.e.
            ["eu am", "tu ai", "el a", "noi am", "voi aţi", "ei au"]
        but for conditional it's supposed to be
            ["eu aş", "tu ai", "el ar", "noi am", "voi aţi", "ei ar"]
        """
        if mood == Mood.Condițional and tense in (Tense.Prezent, "perfect"):
            sub_aux_conj = ["aş", "ai", "ar", "am", "aţi", "ar"]
            for i, c in enumerate(aux_conj):
                pronoun, _ = c.split(" ")
                aux_conj[i] = f"{pronoun} {sub_aux_conj[i]}"
        return aux_conj

    def _add_compound_aux_verb_suffix_if_applicable(
        self, s: str, mood: Mood, tense: Tense
    ) -> str:
        """
        E.g. for Romanian indicativ viitor-ii this appends " fi" to make "eu am să fi avut" etc.
        """
        if (mood == Mood.Indicativ and tense == Tense.Viitor2) or (
            mood == Mood.Condițional and tense == Tense.Perfect
        ):
            return s + " fi"
        elif mood == Mood.Indicativ and tense == Tense.Viitor2Popular:
            return s + " să fi"
        # TODO: Research. Some sources e.g. verbix.com don't include " să"
        # elif mood == Mood.Indicativ and tense == "viitor-1":
        #    return s + " să"
        return s

    def _insert_compound_aux_verb_prefix_if_applicable(
        self, s: str, mood: Mood, tense: Tense
    ) -> str:
        """
        Used by Romanian viitor-1-popular
        "eu o să fac, tu o să faci, ..."
        """
        if mood == Mood.Indicativ and tense == Tense.Viitor1Popular:
            tokens = s.split()
            return tokens[0] + " o să " + tokens[1]
        return s

    def _compound_has_no_primary_verb(self, mood: Mood, tense: Tense) -> bool:
        """Used for Romanian viitor-1-popular"""
        if mood == Mood.Indicativ and tense == Tense.Viitor1Popular:
            return True
        return False

    def _compound_has_no_aux_verb(self, mood: Mood, tense: Tense) -> bool:
        """Used for Romanian conjunctiv perfect"""
        if mood == Mood.Conjunctiv and tense == Tense.Perfect:
            return True
        return False



from unittest.mock import patch

import pytest

from verbecc.src.parsers.conjugation_template import ConjugationTemplate
from verbecc.src.defs.types.exceptions import ConjugationTemplateError
from verbecc.src.defs.types.lang_code import LangCodeISO639_1 as Lang


@patch("lxml.etree._Element")
def test_template_invalid_tag_template(mock_template_elem):
    mock_template_elem.tag.return_value = "not-template"
    with pytest.raises(ConjugationTemplateError):
        template = ConjugationTemplate(Lang.fr, mock_template_elem)


@patch("lxml.etree._Element")
def test_template_invalid_tag_name(mock_template_elem):
    mock_template_elem.get.return_value = "not-name"
    with pytest.raises(ConjugationTemplateError):
        template = ConjugationTemplate(Lang.ca, mock_template_elem)



import pytest
from verbecc.src.conjugator.conjugator import Conjugator
from verbecc.src.defs.types.exceptions import (
    InvalidMoodError,
    InvalidTenseError,
    TemplateNotFoundError,
)
from verbecc.src.defs.constants import config


@pytest.fixture(scope="module")
def cg():
    cg = Conjugator(lang="fr")
    yield cg


def test_get_infinitives(cg):
    infinitives = cg.get_infinitives()
    assert len(infinitives) > 7000
    assert "parler" in infinitives


def test_get_template_names(cg):
    template_names = cg.get_template_names()
    assert len(template_names) >= 146
    assert "aim:er" in template_names


test_verbs = [
    ("manger"),
    ("venir"),
    ("être"),
    ("aller"),
    ("pouvoir"),
    ("finir"),
    ("pleuvoir"),
]


@pytest.mark.parametrize("infinitive", test_verbs)
def test_conjugator_conjugate_basic(cg, infinitive):
    output = cg.conjugate(infinitive)
    assert output


def test_conjugator_predict_conjugation_er_verb_indicative_present(cg):
    if config.ml:
        assert cg.conjugate_mood_tense("ubériser", "indicatif", "présent") == [
            "j'ubérise",
            "tu ubérises",
            "il ubérise",
            "nous ubérisons",
            "vous ubérisez",
            "ils ubérisent",
        ]


def test_conjugator_predict_conjugation_re_verb_indicative_present(cg):
    if config.ml:
        assert cg.conjugate_mood_tense("brettre", "indicatif", "présent") == [
            "je brets",
            "tu brets",
            "il bret",
            "nous brettons",
            "vous brettez",
            "ils brettent",
        ]


def test_conjugator_conjugate_passe_compose_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "indicatif", "passé-composé") == [
        "j'ai mangé",
        "tu as mangé",
        "il a mangé",
        "nous avons mangé",
        "vous avez mangé",
        "ils ont mangé",
    ]


def test_conjugator_conjugate_passe_compose_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "indicatif", "passé-composé") == [
        "je suis allé",
        "tu es allé",
        "il est allé",
        "nous sommes allés",
        "vous êtes allés",
        "ils sont allés",
    ]


def test_conjugator_conjugate_subjonctif_passe_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "subjonctif", "passé") == [
        "que j'aie mangé",
        "que tu aies mangé",
        "qu'il ait mangé",
        "que nous ayons mangé",
        "que vous ayez mangé",
        "qu'ils aient mangé",
    ]


def test_conjugator_conjugate_subjonctif_passe_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "subjonctif", "passé") == [
        "que je sois allé",
        "que tu sois allé",
        "qu'il soit allé",
        "que nous soyons allés",
        "que vous soyez allés",
        "qu'ils soient allés",
    ]


def test_conjugator_conjugate_conditionnel_passe_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "conditionnel", "passé") == [
        "j'aurais mangé",
        "tu aurais mangé",
        "il aurait mangé",
        "nous aurions mangé",
        "vous auriez mangé",
        "ils auraient mangé",
    ]


def test_conjugator_conjugate_conditionnel_passe_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "conditionnel", "passé") == [
        "je serais allé",
        "tu serais allé",
        "il serait allé",
        "nous serions allés",
        "vous seriez allés",
        "ils seraient allés",
    ]


def test_conjugator_conjugate_plusqueparfait_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "indicatif", "plus-que-parfait") == [
        "j'avais mangé",
        "tu avais mangé",
        "il avait mangé",
        "nous avions mangé",
        "vous aviez mangé",
        "ils avaient mangé",
    ]


def test_conjugator_conjugate_plusqueparfait_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "indicatif", "plus-que-parfait") == [
        "j'étais allé",
        "tu étais allé",
        "il était allé",
        "nous étions allés",
        "vous étiez allés",
        "ils étaient allés",
    ]


def test_conjugator_conjugate_subjonctif_plusqueparfait_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "subjonctif", "plus-que-parfait") == [
        "que j'eusse mangé",
        "que tu eusses mangé",
        "qu'il eût mangé",
        "que nous eussions mangé",
        "que vous eussiez mangé",
        "qu'ils eussent mangé",
    ]


def test_conjugator_conjugate_subjonctif_plusqueparfait_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "subjonctif", "plus-que-parfait") == [
        "que je fusse allé",
        "que tu fusses allé",
        "qu'il fût allé",
        "que nous fussions allés",
        "que vous fussiez allés",
        "qu'ils fussent allés",
    ]


def test_conjugator_conjugate_futur_anterieur_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "indicatif", "futur-antérieur") == [
        "j'aurai mangé",
        "tu auras mangé",
        "il aura mangé",
        "nous aurons mangé",
        "vous aurez mangé",
        "ils auront mangé",
    ]


def test_conjugator_conjugate_futur_anterieur_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "indicatif", "futur-antérieur") == [
        "je serai allé",
        "tu seras allé",
        "il sera allé",
        "nous serons allés",
        "vous serez allés",
        "ils seront allés",
    ]


def test_conjugator_conjugate_passe_anterieur_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "indicatif", "passé-antérieur") == [
        "j'eus mangé",
        "tu eus mangé",
        "il eut mangé",
        "nous eûmes mangé",
        "vous eûtes mangé",
        "ils eurent mangé",
    ]


def test_conjugator_conjugate_passe_anterieur_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "indicatif", "passé-antérieur") == [
        "je fus allé",
        "tu fus allé",
        "il fut allé",
        "nous fûmes allés",
        "vous fûtes allés",
        "ils furent allés",
    ]


def test_conjugator_conjugate_imperatif_passe_with_avoir(cg):
    assert cg.conjugate_mood_tense("manger", "imperatif", "imperatif-passé") == [
        "aie mangé",
        "ayons mangé",
        "ayez mangé",
    ]


def test_conjugator_conjugate_imperatif_passe_with_etre(cg):
    assert cg.conjugate_mood_tense("aller", "imperatif", "imperatif-passé") == [
        "sois allé",
        "soyons allés",
        "soyez allés",
    ]


expected_resp_conj_manger = {
    "verb": {
        "infinitive": "manger",
        "predicted": False,
        "pred_score": 1.0,
        "template": "man:ger",
        "translation_en": "eat",
        "stem": "man",
    },
    "moods": {
        "infinitif": {"infinitif-présent": ["manger"]},
        "indicatif": {
            "présent": [
                "je mange",
                "tu manges",
                "il mange",
                "nous mangeons",
                "vous mangez",
                "ils mangent",
            ],
            "imparfait": [
                "je mangeais",
                "tu mangeais",
                "il mangeait",
                "nous mangions",
                "vous mangiez",
                "ils mangeaient",
            ],
            "futur-simple": [
                "je mangerai",
                "tu mangeras",
                "il mangera",
                "nous mangerons",
                "vous mangerez",
                "ils mangeront",
            ],
            "passé-simple": [
                "je mangeai",
                "tu mangeas",
                "il mangea",
                "nous mangeâmes",
                "vous mangeâtes",
                "ils mangèrent",
            ],
            "passé-composé": [
                "j'ai mangé",
                "tu as mangé",
                "il a mangé",
                "nous avons mangé",
                "vous avez mangé",
                "ils ont mangé",
            ],
            "plus-que-parfait": [
                "j'avais mangé",
                "tu avais mangé",
                "il avait mangé",
                "nous avions mangé",
                "vous aviez mangé",
                "ils avaient mangé",
            ],
            "futur-antérieur": [
                "j'aurai mangé",
                "tu auras mangé",
                "il aura mangé",
                "nous aurons mangé",
                "vous aurez mangé",
                "ils auront mangé",
            ],
            "passé-antérieur": [
                "j'eus mangé",
                "tu eus mangé",
                "il eut mangé",
                "nous eûmes mangé",
                "vous eûtes mangé",
                "ils eurent mangé",
            ],
        },
        "conditionnel": {
            "présent": [
                "je mangerais",
                "tu mangerais",
                "il mangerait",
                "nous mangerions",
                "vous mangeriez",
                "ils mangeraient",
            ],
            "passé": [
                "j'aurais mangé",
                "tu aurais mangé",
                "il aurait mangé",
                "nous aurions mangé",
                "vous auriez mangé",
                "ils auraient mangé",
            ],
        },
        "subjonctif": {
            "présent": [
                "que je mange",
                "que tu manges",
                "qu'il mange",
                "que nous mangions",
                "que vous mangiez",
                "qu'ils mangent",
            ],
            "imparfait": [
                "que je mangeasse",
                "que tu mangeasses",
                "qu'il mangeât",
                "que nous mangeassions",
                "que vous mangeassiez",
                "qu'ils mangeassent",
            ],
            "passé": [
                "que j'aie mangé",
                "que tu aies mangé",
                "qu'il ait mangé",
                "que nous ayons mangé",
                "que vous ayez mangé",
                "qu'ils aient mangé",
            ],
            "plus-que-parfait": [
                "que j'eusse mangé",
                "que tu eusses mangé",
                "qu'il eût mangé",
                "que nous eussions mangé",
                "que vous eussiez mangé",
                "qu'ils eussent mangé",
            ],
        },
        "imperatif": {
            "imperatif-présent": ["mange", "mangeons", "mangez"],
            "imperatif-passé": ["aie mangé", "ayons mangé", "ayez mangé"],
        },
        "participe": {
            "participe-présent": ["mangeant"],
            "participe-passé": ["mangé", "mangés", "mangée", "mangées"],
        },
    },
}

expected_resp_conj_pouvoir = {
    "verb": {
        "infinitive": "pouvoir",
        "predicted": False,
        "pred_score": 1.0,
        "template": "p:ouvoir",
        "translation_en": "power",
        "stem": "p",
    },
    "moods": {
        "infinitif": {"infinitif-présent": ["pouvoir"]},
        "indicatif": {
            "présent": [
                "je peux",
                "tu peux",
                "il peut",
                "nous pouvons",
                "vous pouvez",
                "ils peuvent",
            ],
            "imparfait": [
                "je pouvais",
                "tu pouvais",
                "il pouvait",
                "nous pouvions",
                "vous pouviez",
                "ils pouvaient",
            ],
            "futur-simple": [
                "je pourrai",
                "tu pourras",
                "il pourra",
                "nous pourrons",
                "vous pourrez",
                "ils pourront",
            ],
            "passé-simple": [
                "je pus",
                "tu pus",
                "il put",
                "nous pûmes",
                "vous pûtes",
                "ils purent",
            ],
            "passé-composé": [
                "j'ai pu",
                "tu as pu",
                "il a pu",
                "nous avons pu",
                "vous avez pu",
                "ils ont pu",
            ],
            "plus-que-parfait": [
                "j'avais pu",
                "tu avais pu",
                "il avait pu",
                "nous avions pu",
                "vous aviez pu",
                "ils avaient pu",
            ],
            "futur-antérieur": [
                "j'aurai pu",
                "tu auras pu",
                "il aura pu",
                "nous aurons pu",
                "vous aurez pu",
                "ils auront pu",
            ],
            "passé-antérieur": [
                "j'eus pu",
                "tu eus pu",
                "il eut pu",
                "nous eûmes pu",
                "vous eûtes pu",
                "ils eurent pu",
            ],
        },
        "conditionnel": {
            "présent": [
                "je pourrais",
                "tu pourrais",
                "il pourrait",
                "nous pourrions",
                "vous pourriez",
                "ils pourraient",
            ],
            "passé": [
                "j'aurais pu",
                "tu aurais pu",
                "il aurait pu",
                "nous aurions pu",
                "vous auriez pu",
                "ils auraient pu",
            ],
        },
        "subjonctif": {
            "présent": [
                "que je puisse",
                "que tu puisses",
                "qu'il puisse",
                "que nous puissions",
                "que vous puissiez",
                "qu'ils puissent",
            ],
            "imparfait": [
                "que je pusse",
                "que tu pusses",
                "qu'il pût",
                "que nous pussions",
                "que vous pussiez",
                "qu'ils pussent",
            ],
            "passé": [
                "que j'aie pu",
                "que tu aies pu",
                "qu'il ait pu",
                "que nous ayons pu",
                "que vous ayez pu",
                "qu'ils aient pu",
            ],
            "plus-que-parfait": [
                "que j'eusse pu",
                "que tu eusses pu",
                "qu'il eût pu",
                "que nous eussions pu",
                "que vous eussiez pu",
                "qu'ils eussent pu",
            ],
        },
        "imperatif": {"imperatif-présent": [], "imperatif-passé": []},
        "participe": {
            "participe-présent": ["pouvant"],
            "participe-passé": ["pu", "pus", "pue", "pues"],
        },
    },
}

expected_resp_conj_pleuvoir = {
    "verb": {
        "infinitive": "pleuvoir",
        "predicted": False,
        "pred_score": 1.0,
        "template": "pl:euvoir",
        "translation_en": "rain",
        "stem": "pl",
    },
    "moods": {
        "infinitif": {"infinitif-présent": ["pleuvoir"]},
        "indicatif": {
            "présent": ["il pleut", "ils pleuvent"],
            "imparfait": ["il pleuvait", "ils pleuvaient"],
            "futur-simple": ["il pleuvra", "ils pleuvront"],
            "passé-simple": ["il plut", "ils plurent"],
            "passé-composé": ["il a plu", "ils ont plu"],
            "plus-que-parfait": ["il avait plu", "ils avaient plu"],
            "futur-antérieur": ["il aura plu", "ils auront plu"],
            "passé-antérieur": ["il eut plu", "ils eurent plu"],
        },
        "conditionnel": {
            "présent": ["il pleuvrait", "ils pleuvraient"],
            "passé": ["il aurait plu", "ils auraient plu"],
        },
        "subjonctif": {
            "présent": ["qu'il pleuve", "qu'ils pleuvent"],
            "imparfait": ["qu'il plût", "qu'ils plussent"],
            "passé": ["qu'il ait plu", "qu'ils aient plu"],
            "plus-que-parfait": ["qu'il eût plu", "qu'ils eussent plu"],
        },
        "imperatif": {"imperatif-présent": [], "imperatif-passé": []},
        "participe": {
            "participe-présent": ["pleuvant"],
            "participe-passé": ["plu", "plus", "plue", "plues"],
        },
    },
}

expected_resp_conj_se_lever = {
    "verb": {
        "infinitive": "lever",
        "predicted": False,
        "pred_score": 1.0,
        "template": "l:ever",
        "translation_en": "lift",
        "stem": "l",
    },
    "moods": {
        "infinitif": {"infinitif-présent": ["se lever"]},
        "indicatif": {
            "présent": [
                "je me lève",
                "tu te lèves",
                "il se lève",
                "nous nous levons",
                "vous vous levez",
                "ils se lèvent",
            ],
            "imparfait": [
                "je me levais",
                "tu te levais",
                "il se levait",
                "nous nous levions",
                "vous vous leviez",
                "ils se levaient",
            ],
            "futur-simple": [
                "je me lèverai",
                "tu te lèveras",
                "il se lèvera",
                "nous nous lèverons",
                "vous vous lèverez",
                "ils se lèveront",
            ],
            "passé-simple": [
                "je me levai",
                "tu te levas",
                "il se leva",
                "nous nous levâmes",
                "vous vous levâtes",
                "ils se levèrent",
            ],
            "passé-composé": [
                "je me suis levé",
                "tu t'es levé",
                "il s'est levé",
                "nous nous sommes levés",
                "vous vous êtes levés",
                "ils se sont levés",
            ],
            "plus-que-parfait": [
                "je m'étais levé",
                "tu t'étais levé",
                "il s'était levé",
                "nous nous étions levés",
                "vous vous étiez levés",
                "ils s'étaient levés",
            ],
            "futur-antérieur": [
                "je me serai levé",
                "tu te seras levé",
                "il se sera levé",
                "nous nous serons levés",
                "vous vous serez levés",
                "ils se seront levés",
            ],
            "passé-antérieur": [
                "je me fus levé",
                "tu te fus levé",
                "il se fut levé",
                "nous nous fûmes levés",
                "vous vous fûtes levés",
                "ils se furent levés",
            ],
        },
        "conditionnel": {
            "présent": [
                "je me lèverais",
                "tu te lèverais",
                "il se lèverait",
                "nous nous lèverions",
                "vous vous lèveriez",
                "ils se lèveraient",
            ],
            "passé": [
                "je me serais levé",
                "tu te serais levé",
                "il se serait levé",
                "nous nous serions levés",
                "vous vous seriez levés",
                "ils se seraient levés",
            ],
        },
        "subjonctif": {
            "présent": [
                "que je me lève",
                "que tu te lèves",
                "qu'il se lève",
                "que nous nous levions",
                "que vous vous leviez",
                "qu'ils se lèvent",
            ],
            "imparfait": [
                "que je me levasse",
                "que tu te levasses",
                "qu'il se levât",
                "que nous nous levassions",
                "que vous vous levassiez",
                "qu'ils se levassent",
            ],
            "passé": [
                "que je me sois levé",
                "que tu te sois levé",
                "qu'il se soit levé",
                "que nous nous soyons levés",
                "que vous vous soyez levés",
                "qu'ils se soient levés",
            ],
            "plus-que-parfait": [
                "que je me fusse levé",
                "que tu te fusses levé",
                "qu'il se fût levé",
                "que nous nous fussions levés",
                "que vous vous fussiez levés",
                "qu'ils se fussent levés",
            ],
        },
        "imperatif": {
            "imperatif-présent": ["lève-toi", "levons-nous", "levez-vous"],
            "imperatif-passé": [],
        },
        "participe": {
            "participe-présent": ["se levant"],
            "participe-passé": [
                "s'étant levé",
                "s'étant levés",
                "s'étant levée",
                "s'étant levées",
            ],
        },
    },
}


@pytest.mark.parametrize(
    "infinitive,expected_resp",
    [
        ("manger", expected_resp_conj_manger),
        ("pouvoir", expected_resp_conj_pouvoir),
        ("Pouvoir", expected_resp_conj_pouvoir),
        ("pleuvoir", expected_resp_conj_pleuvoir),
        ("Se lever", expected_resp_conj_se_lever),
    ],
)
def test_conjugator_conjugate(cg, infinitive, expected_resp):
    assert cg.conjugate(infinitive) == expected_resp


def test_conjugator_conjugate_invalid_mood(cg):
    with pytest.raises(InvalidMoodError):
        cg.conjugate_mood("manger", "oops")


def test_conjugator_conjugate_invalid_tense(cg):
    with pytest.raises(InvalidTenseError):
        cg.conjugate_mood_tense("manger", "indicatif", "oops")


def test_conjugator_find_template_template_not_found(cg):
    with pytest.raises(TemplateNotFoundError):
        cg.find_template("oops")


@pytest.mark.parametrize(
    "query,expected_resp",
    [
        ("lev", ["lever", "léviger", "levretter"]),
        ("Se lev", ["se lever", "se léviger", "se levretter"]),
        ("s'aim", ["s'aimanter", "s'aimer"]),
    ],
)
def test_conjugator_get_verbs_that_start_with(cg, query, expected_resp):
    assert set(cg.get_verbs_that_start_with(query, max_results=10)) == set(
        expected_resp
    )



from typing import Union, Dict, List

from verbecc.src.defs.types.mood import Mood
from verbecc.src.defs.types.tense import Tense

import json


# PersonConjugation format depends on AlternatesBehavior
# With AlternatesBehavior.All, its a List[str]
# With AlternatesBehvioar.FirstOnly or SecondOnly, it's a str
PersonConjugation = Union[List[str], str]
TenseConjugation = List[PersonConjugation]
MoodConjugation = Dict[Tense, TenseConjugation]
MoodsConjugation = Dict[Mood, MoodConjugation]  # the "moods" section
ConjugationInfoData = Dict[str, Union[str, bool, float]]  # the "verb" section
Conjugation = Dict[str, Union[ConjugationInfoData, MoodsConjugation]]


class ConjugationInfo:
    infinitive: str
    predicted: bool
    pred_score: float
    template: str
    translation_en: str
    stem: str

    def __init__(
        self,
        infinitive: str,
        predicted: bool,
        pred_score: float,
        template: str,
        translation_en: str,
        stem: str,
    ) -> None:
        self.infinitive = infinitive
        self.predicted = predicted
        self.pred_score = pred_score
        self.template = template
        self.translation_en = translation_en
        self.stem = stem

    def __str__(self) -> str:
        return json.dumps(
            self,
            allow_nan=False,
            sort_keys=True,
            indent=4,
        )

    @property
    def data(self) -> ConjugationInfoData:
        return {
            "infinitive": self.infinitive,
            "predicted": self.predicted,
            "pred_score": self.pred_score,
            "template": self.template,
            "translation_en": self.translation_en,
            "stem": self.stem,
        }



from lxml import etree
from typing import List

from verbecc.src.parsers.person_ending import PersonEnding
from verbecc.src.defs.constants import grammar_defines
from verbecc.src.defs.types.tense import Tense
from verbecc.src.defs.types.person import Person
from verbecc.src.defs.types.mood import Mood
from verbecc.src.defs.types.lang_code import LangCodeISO639_1 as Lang
from verbecc.src.defs.constants.localization import xmood


class TenseTemplate:
    """
    Contains PersonEndings for a specific verb template, mood and tense
    Note: The template name and mood is only known by the Mood object
          which owns this Tense.
    Class relationships:
        ConjugationTemplate has many Mood
            Mood has many Tense
                Tense has many PersonEnding
    E.g. aim:er indicatif présent
    name
        the name of the tense, e.g. présent
    tense_elem
        A tense_elem contains one or more <p> (PersonEnding) elems
        Example tense_elem children:
            <p><i>e</i></p>
            <p><i>es</i></p>
            <p><i>e</i></p>
            <p><i>ons</i></p>
            <p><i>ez</i></p>
            <p><i>ent</i></p>
    """

    def __init__(self, lang: Lang, mood: Mood, tense_elem: etree._Element) -> None:
        self.lang = lang
        self.mood = mood
        self.name = tense_elem.tag
        """
        Normally each <p> elem defines six grammatical persons:
            (see grammar_defines.PERSONS)
            [0]= '1s' (je)
            [1]= '2s' (tu)
            [2]= '3s' (il, elle, on)
            [3]= '1p' (nous)
            [4]= '2p' (vous)
            [5]= '3p' (ils, elles)

        The following tenses have all 6 Persons:
          présent, impafait, futur, passé-simple
        These do not:
          infinitive-present has only 1
          imperative-present has 3
            [0]= '2s' e.g. lève-toi
            [2]= '1p' e.g. levons-nous
            [3]= '2p' e.g. levez-vous
          participe-présent has 1
          participe-passé has 4
              [0]= ms
              [1]= mp
              [2]= fs
              [3]= fp

        For some verbs, e.g. être, the past-participle
        is the same for all 4 inflections, so the xml omits them:
            <p><i>été</i></p>
            <p></p>
            <p></p>
            <p></p>
        Workaround: We do not add a PersonEnding unless it contains an <i>

        Also some verbs are impersonal, e.g. pleuvoir, so they don't have
        ending for all pronouns. Just 3rd person singular and plural

            <p></p>
            <p></p>
            <p><i>eut</i></p> e.g. il pleut
            <p></p>
            <p></p>
            <p><i>euvent</i></p> e.g. ils pleuvent (rare, but valid)

        Spanish imperative tenses have 5, i.e. all except 1st person singular

        """
        self.person_endings: List[PersonEnding] = []
        person_num = 0
        for p_elem in tense_elem.findall("p", namespaces=None):
            person = grammar_defines.PERSONS[person_num]
            if self.mood == xmood(self.lang, Mood.en.Imperative):
                person = grammar_defines.IMPERATIVE_PERSONS[self.lang][person_num]
            pe = PersonEnding(p_elem, person)
            person_num += 1
            if len(pe.endings) > 0:
                self.person_endings.append(pe)

    def get_person_ending(self, person: Person) -> PersonEnding:
        for pe in self.person_endings:
            if pe.person == person:
                return pe
        raise ValueError(
            "TenseTemplate '{}' has no PersonEnding for person '{}'".format(
                self.name, person
            )
        )

    def __repr__(self) -> str:
        return "person_endings={}".format(self.person_endings)



import pytest

from verbecc.src.defs.constants.localization import xmood, xtense


@pytest.mark.parametrize(
    "lang,mood,expected",
    [
        ("es", "subjunctive", "subjuntivo"),
        ("fr", "indicative", "indicatif"),
        ("ca", "indicative", "indicatiu"),
    ],
)
def test_xmood(lang, mood, expected):
    assert xmood(lang, mood) == expected


@pytest.mark.parametrize(
    "lang,tense,expected",
    [
        ("es", "present", "presente"),
        ("es", "gerund", "gerundio"),
        ("fr", "gerund", "participe-présent"),
        ("fr", "present", "présent"),
    ],
)
def test_xtense(lang, tense, expected):
    assert xtense(lang, tense) == expected



import logging

from verbecc.src.defs.types.gender import Gender
from verbecc.src.defs.types.mood import Mood
from verbecc.src.defs.types.tense import Tense
from verbecc.src.defs.types.conjugation import ConjugationInfo
from verbecc.src.defs.constants.config import DEVEL_MODE

logging_level = logging.CRITICAL + 1  # effectively disables logging
if DEVEL_MODE:
    logging_level = logging.DEBUG

logging.basicConfig(
    level=logging_level,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[logging.FileHandler("verbecc.log"), logging.StreamHandler()],
)

logger = logging.getLogger(__name__)

import copy
from typing import cast, List

from verbecc.src.parsers.verb import Verb
from verbecc.src.parsers.tense_template import TenseTemplate
from verbecc.src.parsers.conjugation_template import ConjugationTemplate
from verbecc.src.inflectors.inflector_factory import InflectorFactory
from verbecc.src.conjugator.conjugation_object import ConjugationObjects
from verbecc.src.defs.types.person import Person
from verbecc.src.defs.types.exceptions import (
    VerbNotFoundError,
    InvalidMoodError,
    InvalidTenseError,
)
from verbecc.src.defs.types.conjugation import (
    PersonConjugation,
    TenseConjugation,
    MoodConjugation,
    MoodsConjugation,
    Conjugation,
)
from verbecc.src.defs.types.lang_specific_options import (
    LangSpecificOptions,
)
from verbecc.src.conjugator.conjugation_object import ConjugationObjects
from verbecc.src.defs.types.lang_code import LangCodeISO639_1
from verbecc.src.utils.string_utils import strip_accents
from verbecc.src.defs.types.alternates_behavior import AlternatesBehavior


class Conjugator:
    """
    Conjugator encapsulates all conjugation logic that is not language-specific.
    Conjugator uses a concrete instance of Inflector for all language-specific
    conjugation logic.
    """

    def __init__(self, lang: LangCodeISO639_1) -> None:
        self._inflector = InflectorFactory.make_inflector(lang)

    def conjugate(
        self,
        infinitive: str,
        include_alternates: bool = False,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> Conjugation:
        """
        :param infinitive: the infinitive form of the verb to conjugate
        :type infinitive: str

        :param include_alternates: whether to include alternate conjugations
        e.g. the Catalan verbs ser/ésser have alternate conjugations in
        the conditional tense and the participle
        See test_inflector_ca.test_inflector_conjugate_with_alternates
        :type include_alternates: bool

        :param gender: controls gender of third-person singular and plural
        :type gender: Gender

        :param conjugate_pronouns: if True, verbecc will conjugate the pronoun together with
        its inflected form, e.g. for the French verb apprendre, for the first-person singular
        present tense you'd get "j'apprends" if True or "apprends" if False.
        :type conjugate_pronouns: bool

        :param lang_specific_options: options specific to certain languages.
        :type lang_specific_options: LangSpecificOptions
        """
        alternates_behavior = AlternatesBehavior.FirstOnly
        if include_alternates:
            alternates_behavior = AlternatesBehavior.All
        co = self._get_conj_obs(infinitive)
        moods: MoodsConjugation = {}
        for mood, _ in co.template.mood_templates.items():
            moods[mood] = self._conjugate_mood(
                co,
                mood,
                alternates_behavior,
                gender,
                conjugate_pronouns,
                lang_specific_options,
            )
        return {
            "verb": ConjugationInfo(
                co.verb.infinitive,
                co.verb.predicted,
                co.verb.pred_score,
                co.verb.template,
                co.verb.translation_en,
                co.verb_stem,
            ).data,
            "moods": moods,
        }

    def conjugate_mood(
        self,
        infinitive: str,
        mood: Mood,
        alternates_behavior: AlternatesBehavior = AlternatesBehavior.FirstOnly,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> MoodConjugation:
        co = self._get_conj_obs(infinitive)
        return self._conjugate_mood(
            co, mood, alternates_behavior, gender, conjugate_pronouns
        )

    def _get_conj_obs(self, infinitive: str) -> ConjugationObjects:
        infinitive = infinitive.lower()
        is_reflexive, infinitive = self._inflector._split_reflexive(infinitive)
        if is_reflexive and not self._inflector._verb_can_be_reflexive(infinitive):
            raise VerbNotFoundError("Verb cannot be reflexive")
        verb = self.find_verb_by_infinitive(infinitive)
        template = self.find_template(verb.template)
        verb_stem = self._inflector._get_verb_stem_from_template_name(
            verb.infinitive, template.name
        )
        return ConjugationObjects(infinitive, verb, template, verb_stem, is_reflexive)

    def get_verbs(self) -> List[Verb]:
        return self._inflector.get_verbs()

    def get_infinitives(self) -> List[str]:
        return self._inflector.get_infinitives()

    def get_templates(self) -> List[ConjugationTemplate]:
        return self._inflector.get_templates()

    def get_template_names(self) -> List[str]:
        return self._inflector.get_template_names()

    def find_verb_by_infinitive(self, infinitive: str) -> Verb:
        return self._inflector.find_verb_by_infinitive(infinitive)

    def find_template(self, name: str) -> ConjugationTemplate:
        return self._inflector.find_template(name)

    def get_verbs_that_start_with(self, query: str, max_results: int) -> List[str]:
        return self._inflector.get_verbs_that_start_with(query, max_results)

    def conjugate_mood_tense(
        self,
        infinitive: str,
        mood: Mood,
        tense: Tense,
        alternates_behavior: AlternatesBehavior = AlternatesBehavior.FirstOnly,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> TenseConjugation:
        co = self._get_conj_obs(infinitive)
        return self._conjugate_mood_tense(
            co,
            mood,
            tense,
            alternates_behavior=alternates_behavior,
            gender=gender,
            conjugate_pronouns=conjugate_pronouns,
            lang_specific_options=lang_specific_options,
        )

    def _conjugate_mood(
        self,
        co: ConjugationObjects,
        mood: Mood,
        alternates_behavior: AlternatesBehavior,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> MoodConjugation:
        if mood not in co.template.mood_templates.keys():
            raise InvalidMoodError
        ret = {}
        ret.update(
            self._get_simple_conjugations_for_mood(
                co,
                mood,
                alternates_behavior,
                gender,
                conjugate_pronouns,
                lang_specific_options,
            )
        )
        ret.update(
            self._get_compound_conjugations_for_mood(
                co,
                mood,
                alternates_behavior,
                gender,
                conjugate_pronouns,
                lang_specific_options,
            )
        )
        return ret

    def _conjugate_mood_tense(
        self,
        co: ConjugationObjects,
        mood: Mood,
        tense: Tense,
        alternates_behavior: AlternatesBehavior,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> TenseConjugation:
        """
        :param gender: controls gender of third-person singular and plural
        pronouns, if conjugate_pronouns is enabled. Otherwise ignored.
        """
        comp_conj_map = self._inflector._get_compound_conjugations_aux_verb_map()
        if mood in comp_conj_map and tense in comp_conj_map[mood]:
            aux_mood, aux_tense = comp_conj_map[mood][tense]
            return self._conjugate_compound(
                co,
                mood,
                tense,
                aux_mood,
                aux_tense,
                self._inflector._auxilary_verb_uses_alternate_conjugation(tense),
                alternates_behavior,
                gender=gender,
                conjugate_pronouns=conjugate_pronouns,
                lang_specific_options=lang_specific_options,
            )
        else:
            mood_template = co.template.mood_templates[mood]
            if tense not in mood_template.tense_templates:
                raise InvalidTenseError
            tense_template = mood_template.tense_templates[tense]
            return self._conjugate_simple_mood_tense(
                co.verb_stem,
                mood,
                tense,
                tense_template,
                is_reflexive=co.is_reflexive,
                alternates_behavior=alternates_behavior,
                gender=gender,
                modify_stem_strip_accents=bool(
                    co.template.modify_stem == "strip-accents"
                ),
                conjugate_pronouns=conjugate_pronouns,
                lang_specific_options=lang_specific_options,
            )

    def _get_simple_conjugations_for_mood(
        self,
        co: ConjugationObjects,
        mood: Mood,
        alternates_behavior: AlternatesBehavior,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> MoodConjugation:
        ret = {}
        mood_template = co.template.mood_templates[mood]
        for tense in mood_template.tense_templates:
            ret[tense] = self._conjugate_mood_tense(
                co,
                mood,
                tense,
                alternates_behavior,
                gender,
                conjugate_pronouns=conjugate_pronouns,
                lang_specific_options=lang_specific_options,
            )
        return ret

    def _get_compound_conjugations_for_mood(
        self,
        co: ConjugationObjects,
        mood: Mood,
        alternates_behavior: AlternatesBehavior,
        gender: Gender,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> MoodConjugation:
        ret = {}
        comp_conj_map = self._inflector._get_compound_conjugations_aux_verb_map()
        if mood in comp_conj_map:
            for tense in comp_conj_map[mood]:
                ret[tense] = self._conjugate_mood_tense(
                    co,
                    mood,
                    tense,
                    alternates_behavior,
                    gender,
                    conjugate_pronouns=conjugate_pronouns,
                    lang_specific_options=lang_specific_options,
                )
        return ret

    def _conjugate_compound(
        self,
        co: ConjugationObjects,
        mood: Mood,
        tense: Tense,
        aux_mood: Mood,
        aux_tense: Tense,
        aux_uses_alternate: bool,
        alternates_behavior: AlternatesBehavior,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        lang_specific_options: LangSpecificOptions = None,
    ) -> TenseConjugation:
        """
        :param gender: controls gender of third-person singular and plural
        pronouns, if conjugate_pronouns is enabled. Otherwise ignored.
        """
        ret = []
        if self._inflector._compound_conjugation_not_applicable(
            co.is_reflexive, mood, aux_tense
        ):
            return ret
        persons_mood = mood
        if mood not in co.template.mood_templates.keys():
            persons_mood = self._inflector._get_indicative_mood()
        persons = [
            pe.person
            for pe in co.template.mood_templates[persons_mood]
            .tense_templates[aux_tense]
            .person_endings
        ]
        aux_verb = self._inflector._get_auxilary_verb(co, mood, tense)
        aux_co = self._get_conj_obs(aux_verb)
        aux_tense_template = copy.deepcopy(
            aux_co.template.mood_templates[aux_mood].tense_templates[aux_tense]
        )
        aux_person_endings = []
        for pe in aux_tense_template.person_endings:
            if pe.person in persons:
                aux_person_endings.append(pe)
        aux_tense_template.person_endings = aux_person_endings
        aux_alternates_behavior = AlternatesBehavior.FirstOnly
        if aux_uses_alternate:
            aux_alternates_behavior = AlternatesBehavior.SecondOnly
        aux_conj = self._conjugate_simple_mood_tense(
            aux_co.verb_stem,
            "",
            aux_tense,
            aux_tense_template,
            co.is_reflexive,
            aux_alternates_behavior,
            gender=gender,
            conjugate_pronouns=conjugate_pronouns,
            lang_specific_options=lang_specific_options,
        )
        # cast below is safe because we're not using AlternatesBehavior.All
        aux_conj_scalar_list = cast(List[str], aux_conj)
        # need to skip conjugating primary verb for certain tenses e.g. romanian viitor-1
        ret = self._conjugate_compound_primary_verb(
            co,
            mood,
            tense,
            persons,
            aux_verb,
            aux_conj_scalar_list,
            alternates_behavior,
            gender,
        )
        if mood == self._inflector._get_subjunctive_mood():
            if alternates_behavior == AlternatesBehavior.All:
                ret = [
                    [
                        self._inflector._add_subjunctive_relative_pronoun(i, tense)
                        for i in j
                    ]
                    for j in ret
                ]
            else:
                # cast is safe because we're not using AlternatesBehavior.All
                ret = cast(List[str], ret)
                ret = [
                    self._inflector._add_subjunctive_relative_pronoun(i, tense)
                    for i in ret
                ]
        return cast(TenseConjugation, ret)

    def _conjugate_compound_primary_verb(
        self,
        co: ConjugationObjects,
        mood: Mood,
        tense: Tense,
        persons: List[Person],
        aux_verb: str,
        aux_conj: List[str],
        alternates_behavior: AlternatesBehavior,
        gender: Gender = Gender.m,
        lang_specific_options: LangSpecificOptions = None,
    ) -> TenseConjugation:
        """
        Forms a compound conjugation composed of an auxiliary verb (aka helping verb)
        conjugation and a primary verb, typically the participle tense.
        Typically the primary verb is a participle but there are exceptions e.g.
        the Romanian indicativ viitor-1 uses the inifitive form instead of the participle

        E.g. in the French conjugation "j'ai parlé", "ai" is the conjugated form of the
        auxiliary verb "avoir" and "parlé" is the participle tense of the primary verb "parler".
        With avoir, the participle is not inflected, it's always "parlé".

        With être, however, the participle is inflected (modified based on gender and number). E.g.:
        je suis allé, tu es allé, il est allé, nous sommes allé(e)s, vous êtes allé(e)s, ils/elles sont allé(e)s

        :param gender: controls gender of third-person singular and plural
        pronouns, if conjugate_pronouns is enabled. Otherwise ignored.
        """
        ret: List[str] = []

        p_conj = []
        # the Romanian indicativ viitor-1 uses the inifitive form instead of the participle
        if self._inflector._compound_primary_verb_conjugation_uses_infinitive(
            mood, tense
        ):
            p_conj = [co.infinitive]
        else:
            p_mood = self._inflector._get_participle_mood()
            p_tense = self._inflector._get_participle_tense()
            p_conj = self._conjugate_simple_mood_tense(
                co.verb_stem,
                p_mood,
                p_tense,
                co.template.mood_templates[p_mood].tense_templates[p_tense],
                False,
                AlternatesBehavior.FirstOnly,
                gender=gender,
            )
            # cast is safe since we're not using AlternatesBehavior.All
            p_conj = cast(List[str], p_conj)

        if not self._inflector._is_auxilary_verb_inflected(aux_verb):
            # participle is not inflected, e.g. French passé composé with avoir
            # where aux_verb = "avoir"
            # e.g. j'ai parlé, tu as parlé, il a parlé, nous avons parlé, vous avez parlé, ils ont parlé

            # special case: Romanian conjunctiv perfect
            # TODO: Refactor further
            if self._inflector._compound_has_no_aux_verb(mood, tense):
                participle = p_conj[0]
                for i, c in enumerate(aux_conj):
                    pronoun, _ = aux_conj[i].split()
                    aux_conj[i] = pronoun + " " + participle

            # Normally Romanian aux_conj would be the indicativ prezent tense of avea i.e.
            # ["eu am", "tu ai", "el a", "noi am", "voi aţi", "ei au"]
            # but for conditional it's supposed to be
            # ["eu aş", "tu ai", "el ar", "noi am", "voi aţi", "ei ar"]
            aux_conj = self._inflector._modify_aux_verb_conj_if_applicable(
                aux_conj, mood, tense
            )

            # for Romanian insert " o să " when appropriate
            # e.g. "eu o să face, tu o să faci, ..."
            aux_conj = [
                self._inflector._insert_compound_aux_verb_prefix_if_applicable(
                    i, mood, tense
                )
                for i in aux_conj
            ]

            # for Romanian append " fi", " să fi" etc. when appropriate
            aux_conj = [
                self._inflector._add_compound_aux_verb_suffix_if_applicable(
                    i, mood, tense
                )
                for i in aux_conj
            ]

            # Compound verb conjugation is usually this:
            # {aux_conj(pronoun + aux_conj)} + {compound_suffix} + {primary_conj}
            # where primary_conj is usually the participle of the aux_verb
            #
            # Notable exceptions:
            # 1.
            # The Romanian viitor-1-popular doesn't have a "primary verb", at least not the
            # way it's currently implemented in verbecc. This function just calls the inflector
            # function that adds the " o să" compound suffix.
            # e.g. to get the indicativ viitor-1-popular of the verb 'face' i.e.
            # "eu o să fac, tu o să faci, ..."
            # the verb is coming from aux_conj and p_conj isn't used
            # 2.
            # The Romanian conjunctiv present e.g. 'eu să fi făcut'
            # Doesn't use aux_conj, or rather sets aux_conj = p_conj
            # (see above)

            if self._inflector._compound_has_no_primary_verb(
                mood, tense
            ) or self._inflector._compound_has_no_aux_verb(mood, tense):
                ret = aux_conj
            else:
                for hv in aux_conj:
                    p = "-"
                    if len(p_conj):
                        p = p_conj[0]
                    else:
                        logger.warning(
                            "(aux verb not inflected) primary (participle) conjugation is empty: co=%s p_mood=%s p_tense=%s",
                            co,
                            p_mood,
                            p_tense,
                        )
                    hv = self._inflector._get_alternate_hv_inflection(hv)
                    ret.append(hv + " " + p)
        else:
            # participle is inflected, e.g. French passé composé with être
            # where aux_verb = "être"
            # e.g. je suis allé, tu es allé, il est allé, nous sommes allé(e)s, vous êtes allé(e)s, ils/elles sont allé(e)s
            # or Italian verbs conjugated with essere

            for i, hv in enumerate(aux_conj):
                participle_inflection = (
                    self._inflector._get_default_participle_inflection_for_person(
                        persons[i], gender
                    )
                )
                p = "-"
                participle_idx = (
                    self._inflector._get_participle_index_for_participle_inflection(
                        participle_inflection
                    )
                )
                if len(p_conj) > participle_idx:
                    p = p_conj[participle_idx]
                else:
                    logger.warning(
                        "(aux verb inflected) primary (participle) conjugation is empty: co=%s p_mood=%s t_tense=%s",
                        co,
                        p_mood,
                        p_tense,
                    )
                ret.append(hv + " " + p)
        if alternates_behavior == AlternatesBehavior.All:
            return [[i] for i in ret]
        else:
            return cast(TenseConjugation, ret)

    def _conjugate_simple_mood_tense(
        self,
        verb_stem: str,
        mood: Mood,
        tense: Tense,
        tense_template: TenseTemplate,
        is_reflexive: bool = False,
        alternates_behavior: AlternatesBehavior = AlternatesBehavior.FirstOnly,
        gender: Gender = Gender.m,
        conjugate_pronouns: bool = True,
        modify_stem_strip_accents: bool = False,
        lang_specific_options: LangSpecificOptions = None,
    ) -> TenseConjugation:
        """
        :param gender: controls gender of third-person singular and plural
        pronouns, if conjugate_pronouns is enabled. Otherwise ignored.
        """
        if modify_stem_strip_accents and mood != self._inflector._get_infinitive_mood():
            verb_stem = strip_accents(verb_stem)
        ret = []
        tense = tense_template.name
        if (
            tense in self._inflector._get_tenses_conjugated_without_pronouns()
            or not conjugate_pronouns
        ):
            for person_ending in tense_template.person_endings:
                person_ending = self._inflector.modify_person_ending_if_applicable(
                    person_ending,
                    mood,
                    tense,
                    tense_template,
                    lang_specific_options,
                )
                person_conjugation: PersonConjugation = []
                endings: List[str] = []
                if alternates_behavior == AlternatesBehavior.FirstOnly:
                    endings.append(person_ending.get_ending())
                elif alternates_behavior == AlternatesBehavior.SecondOnly:
                    endings.append(person_ending.get_alternate_ending_if_available())
                else:  # default: AlernatesBehavior.All
                    endings.extend(person_ending.get_endings())
                # there may be one or more alternate endings
                for ending in endings:
                    s = self._inflector._add_present_participle_if_applicable(
                        "", is_reflexive, tense
                    )
                    if ending != "-":
                        s += self._inflector._combine_verb_stem_and_ending(
                            verb_stem, ending
                        )
                    else:
                        s += ending
                    if ending != "-":
                        s = self._inflector._add_reflexive_pronoun_or_pronoun_suffix_if_applicable(
                            s,
                            is_reflexive,
                            mood,
                            tense,
                            person_ending.get_person(),
                        )
                    if ending != "-":
                        s = self._inflector._add_adverb_if_applicable(s, mood, tense)
                    person_conjugation.append(s)
                if alternates_behavior == AlternatesBehavior.All:
                    ret.append(list(person_conjugation))
                elif (
                    alternates_behavior == AlternatesBehavior.SecondOnly
                    and len(person_conjugation) > 1
                ):
                    ret.append(person_conjugation[1])
                else:
                    ret.append(person_conjugation[0])
        else:
            for person_ending in tense_template.person_endings:
                person_ending = self._inflector.modify_person_ending_if_applicable(
                    person_ending,
                    mood,
                    tense,
                    tense_template,
                    lang_specific_options,
                )
                # There will be at least one conjugation per person-ending and
                # potentially one or more alternate conjugations
                person_conjugation: PersonConjugation = []
                for ending in person_ending.get_endings():
                    pronoun = self._inflector.get_default_pronoun(
                        person=person_ending.get_person(),
                        gender=gender,
                        is_reflexive=is_reflexive,
                        lang_specific_options=lang_specific_options,
                    )
                    s = "-"
                    if ending != "-":
                        conj = self._inflector._combine_verb_stem_and_ending(
                            verb_stem, ending
                        )
                        s = self._inflector._combine_pronoun_and_conj(pronoun, conj)
                        if mood == self._inflector._get_subjunctive_mood():
                            s = self._inflector._add_subjunctive_relative_pronoun(
                                s, tense
                            )
                    person_conjugation.append(s)
                if alternates_behavior == AlternatesBehavior.All:
                    ret.append(list(person_conjugation))
                elif (
                    alternates_behavior == AlternatesBehavior.SecondOnly
                    and len(person_conjugation) > 1
                ):
                    ret.append(person_conjugation[1])
                else:
                    ret.append(person_conjugation[0])
        return ret



from verbecc.src.defs.types.lang_specific_options import (
    LangSpecificOptions,
)
from verbecc.src.defs.types.lang.es.voseo_options import VoseoOptions


class LangSpecificOptionsEs(LangSpecificOptions):

    def __init__(self, voseo_options: VoseoOptions = VoseoOptions.NoVoseo) -> None:
        super().__init__()
        self._voseo_options = voseo_options

    @property
    def voseo_options(self) -> VoseoOptions:
        return self._voseo_options
